# Generated by Django 5.2 on 2024-11-14 21:08

import django.db.models.deletion
import lamindb.base.fields
import lamindb.base.users
import lamindb.models
from django.db import migrations

import bionty.ids


class Migration(migrations.Migration):
    dependencies = [
        (
            "bionty",
            "0040_rename_feature_ref_is_symbol_artifactgene_feature_ref_is_name_and_more",
        ),
        (
            "lamindb",
            "0069_alter_artifact__accessor_alter_artifact__hash_type_and_more",
        ),
    ]

    operations = [
        migrations.AlterField(
            model_name="artifactcellline",
            name="artifact",
            field=lamindb.base.fields.ForeignKey(
                blank=True,
                on_delete=django.db.models.deletion.CASCADE,
                related_name="links_cell_line",
                to="lamindb.artifact",
            ),
        ),
        migrations.AlterField(
            model_name="artifactcellline",
            name="cellline",
            field=lamindb.base.fields.ForeignKey(
                blank=True,
                on_delete=django.db.models.deletion.PROTECT,
                related_name="links_artifact",
                to="bionty.cellline",
            ),
        ),
        migrations.AlterField(
            model_name="artifactcellline",
            name="created_at",
            field=lamindb.base.fields.DateTimeField(auto_now_add=True, db_index=True),
        ),
        migrations.AlterField(
            model_name="artifactcellline",
            name="created_by",
            field=lamindb.base.fields.ForeignKey(
                blank=True,
                default=lamindb.base.users.current_user_id,
                on_delete=django.db.models.deletion.PROTECT,
                related_name="+",
                to="lamindb.user",
            ),
        ),
        migrations.AlterField(
            model_name="artifactcellline",
            name="feature",
            field=lamindb.base.fields.ForeignKey(
                blank=True,
                default=None,
                null=True,
                on_delete=django.db.models.deletion.PROTECT,
                related_name="links_artifactcellline",
                to="lamindb.feature",
            ),
        ),
        migrations.AlterField(
            model_name="artifactcellline",
            name="feature_ref_is_name",
            field=lamindb.base.fields.BooleanField(blank=True, default=None, null=True),
        ),
        migrations.AlterField(
            model_name="artifactcellline",
            name="label_ref_is_name",
            field=lamindb.base.fields.BooleanField(blank=True, default=None, null=True),
        ),
        migrations.AlterField(
            model_name="artifactcellline",
            name="run",
            field=lamindb.base.fields.ForeignKey(
                blank=True,
                default=lamindb.models.current_run,
                null=True,
                on_delete=django.db.models.deletion.PROTECT,
                related_name="+",
                to="lamindb.run",
            ),
        ),
        migrations.AlterField(
            model_name="artifactcellmarker",
            name="artifact",
            field=lamindb.base.fields.ForeignKey(
                blank=True,
                on_delete=django.db.models.deletion.CASCADE,
                related_name="links_cell_marker",
                to="lamindb.artifact",
            ),
        ),
        migrations.AlterField(
            model_name="artifactcellmarker",
            name="cellmarker",
            field=lamindb.base.fields.ForeignKey(
                blank=True,
                on_delete=django.db.models.deletion.PROTECT,
                related_name="links_artifact",
                to="bionty.cellmarker",
            ),
        ),
        migrations.AlterField(
            model_name="artifactcellmarker",
            name="created_at",
            field=lamindb.base.fields.DateTimeField(auto_now_add=True, db_index=True),
        ),
        migrations.AlterField(
            model_name="artifactcellmarker",
            name="created_by",
            field=lamindb.base.fields.ForeignKey(
                blank=True,
                default=lamindb.base.users.current_user_id,
                on_delete=django.db.models.deletion.PROTECT,
                related_name="+",
                to="lamindb.user",
            ),
        ),
        migrations.AlterField(
            model_name="artifactcellmarker",
            name="feature",
            field=lamindb.base.fields.ForeignKey(
                blank=True,
                default=None,
                null=True,
                on_delete=django.db.models.deletion.PROTECT,
                related_name="links_artifactcellmarker",
                to="lamindb.feature",
            ),
        ),
        migrations.AlterField(
            model_name="artifactcellmarker",
            name="feature_ref_is_name",
            field=lamindb.base.fields.BooleanField(blank=True, default=None, null=True),
        ),
        migrations.AlterField(
            model_name="artifactcellmarker",
            name="label_ref_is_name",
            field=lamindb.base.fields.BooleanField(blank=True, default=None, null=True),
        ),
        migrations.AlterField(
            model_name="artifactcellmarker",
            name="run",
            field=lamindb.base.fields.ForeignKey(
                blank=True,
                default=lamindb.models.current_run,
                null=True,
                on_delete=django.db.models.deletion.PROTECT,
                related_name="+",
                to="lamindb.run",
            ),
        ),
        migrations.AlterField(
            model_name="artifactcelltype",
            name="artifact",
            field=lamindb.base.fields.ForeignKey(
                blank=True,
                on_delete=django.db.models.deletion.CASCADE,
                related_name="links_cell_type",
                to="lamindb.artifact",
            ),
        ),
        migrations.AlterField(
            model_name="artifactcelltype",
            name="celltype",
            field=lamindb.base.fields.ForeignKey(
                blank=True,
                on_delete=django.db.models.deletion.PROTECT,
                related_name="links_artifact",
                to="bionty.celltype",
            ),
        ),
        migrations.AlterField(
            model_name="artifactcelltype",
            name="created_at",
            field=lamindb.base.fields.DateTimeField(auto_now_add=True, db_index=True),
        ),
        migrations.AlterField(
            model_name="artifactcelltype",
            name="created_by",
            field=lamindb.base.fields.ForeignKey(
                blank=True,
                default=lamindb.base.users.current_user_id,
                on_delete=django.db.models.deletion.PROTECT,
                related_name="+",
                to="lamindb.user",
            ),
        ),
        migrations.AlterField(
            model_name="artifactcelltype",
            name="feature",
            field=lamindb.base.fields.ForeignKey(
                blank=True,
                default=None,
                null=True,
                on_delete=django.db.models.deletion.PROTECT,
                related_name="links_artifactcelltype",
                to="lamindb.feature",
            ),
        ),
        migrations.AlterField(
            model_name="artifactcelltype",
            name="feature_ref_is_name",
            field=lamindb.base.fields.BooleanField(blank=True, default=None, null=True),
        ),
        migrations.AlterField(
            model_name="artifactcelltype",
            name="label_ref_is_name",
            field=lamindb.base.fields.BooleanField(blank=True, default=None, null=True),
        ),
        migrations.AlterField(
            model_name="artifactcelltype",
            name="run",
            field=lamindb.base.fields.ForeignKey(
                blank=True,
                default=lamindb.models.current_run,
                null=True,
                on_delete=django.db.models.deletion.PROTECT,
                related_name="+",
                to="lamindb.run",
            ),
        ),
        migrations.AlterField(
            model_name="artifactdevelopmentalstage",
            name="artifact",
            field=lamindb.base.fields.ForeignKey(
                blank=True,
                on_delete=django.db.models.deletion.CASCADE,
                related_name="links_developmental_stage",
                to="lamindb.artifact",
            ),
        ),
        migrations.AlterField(
            model_name="artifactdevelopmentalstage",
            name="created_at",
            field=lamindb.base.fields.DateTimeField(auto_now_add=True, db_index=True),
        ),
        migrations.AlterField(
            model_name="artifactdevelopmentalstage",
            name="created_by",
            field=lamindb.base.fields.ForeignKey(
                blank=True,
                default=lamindb.base.users.current_user_id,
                on_delete=django.db.models.deletion.PROTECT,
                related_name="+",
                to="lamindb.user",
            ),
        ),
        migrations.AlterField(
            model_name="artifactdevelopmentalstage",
            name="developmentalstage",
            field=lamindb.base.fields.ForeignKey(
                blank=True,
                on_delete=django.db.models.deletion.PROTECT,
                related_name="links_artifact",
                to="bionty.developmentalstage",
            ),
        ),
        migrations.AlterField(
            model_name="artifactdevelopmentalstage",
            name="feature",
            field=lamindb.base.fields.ForeignKey(
                blank=True,
                default=None,
                null=True,
                on_delete=django.db.models.deletion.PROTECT,
                related_name="links_artifactdevelopmentalstage",
                to="lamindb.feature",
            ),
        ),
        migrations.AlterField(
            model_name="artifactdevelopmentalstage",
            name="feature_ref_is_name",
            field=lamindb.base.fields.BooleanField(blank=True, default=None, null=True),
        ),
        migrations.AlterField(
            model_name="artifactdevelopmentalstage",
            name="label_ref_is_name",
            field=lamindb.base.fields.BooleanField(blank=True, default=None, null=True),
        ),
        migrations.AlterField(
            model_name="artifactdevelopmentalstage",
            name="run",
            field=lamindb.base.fields.ForeignKey(
                blank=True,
                default=lamindb.models.current_run,
                null=True,
                on_delete=django.db.models.deletion.PROTECT,
                related_name="+",
                to="lamindb.run",
            ),
        ),
        migrations.AlterField(
            model_name="artifactdisease",
            name="artifact",
            field=lamindb.base.fields.ForeignKey(
                blank=True,
                on_delete=django.db.models.deletion.CASCADE,
                related_name="links_disease",
                to="lamindb.artifact",
            ),
        ),
        migrations.AlterField(
            model_name="artifactdisease",
            name="created_at",
            field=lamindb.base.fields.DateTimeField(auto_now_add=True, db_index=True),
        ),
        migrations.AlterField(
            model_name="artifactdisease",
            name="created_by",
            field=lamindb.base.fields.ForeignKey(
                blank=True,
                default=lamindb.base.users.current_user_id,
                on_delete=django.db.models.deletion.PROTECT,
                related_name="+",
                to="lamindb.user",
            ),
        ),
        migrations.AlterField(
            model_name="artifactdisease",
            name="disease",
            field=lamindb.base.fields.ForeignKey(
                blank=True,
                on_delete=django.db.models.deletion.PROTECT,
                related_name="links_artifact",
                to="bionty.disease",
            ),
        ),
        migrations.AlterField(
            model_name="artifactdisease",
            name="feature",
            field=lamindb.base.fields.ForeignKey(
                blank=True,
                default=None,
                null=True,
                on_delete=django.db.models.deletion.PROTECT,
                related_name="links_artifactdisease",
                to="lamindb.feature",
            ),
        ),
        migrations.AlterField(
            model_name="artifactdisease",
            name="feature_ref_is_name",
            field=lamindb.base.fields.BooleanField(blank=True, default=None, null=True),
        ),
        migrations.AlterField(
            model_name="artifactdisease",
            name="label_ref_is_name",
            field=lamindb.base.fields.BooleanField(blank=True, default=None, null=True),
        ),
        migrations.AlterField(
            model_name="artifactdisease",
            name="run",
            field=lamindb.base.fields.ForeignKey(
                blank=True,
                default=lamindb.models.current_run,
                null=True,
                on_delete=django.db.models.deletion.PROTECT,
                related_name="+",
                to="lamindb.run",
            ),
        ),
        migrations.AlterField(
            model_name="artifactethnicity",
            name="artifact",
            field=lamindb.base.fields.ForeignKey(
                blank=True,
                on_delete=django.db.models.deletion.CASCADE,
                related_name="links_ethnicity",
                to="lamindb.artifact",
            ),
        ),
        migrations.AlterField(
            model_name="artifactethnicity",
            name="created_at",
            field=lamindb.base.fields.DateTimeField(auto_now_add=True, db_index=True),
        ),
        migrations.AlterField(
            model_name="artifactethnicity",
            name="created_by",
            field=lamindb.base.fields.ForeignKey(
                blank=True,
                default=lamindb.base.users.current_user_id,
                on_delete=django.db.models.deletion.PROTECT,
                related_name="+",
                to="lamindb.user",
            ),
        ),
        migrations.AlterField(
            model_name="artifactethnicity",
            name="ethnicity",
            field=lamindb.base.fields.ForeignKey(
                blank=True,
                on_delete=django.db.models.deletion.PROTECT,
                related_name="links_artifact",
                to="bionty.ethnicity",
            ),
        ),
        migrations.AlterField(
            model_name="artifactethnicity",
            name="feature",
            field=lamindb.base.fields.ForeignKey(
                blank=True,
                default=None,
                null=True,
                on_delete=django.db.models.deletion.PROTECT,
                related_name="links_artifactethnicity",
                to="lamindb.feature",
            ),
        ),
        migrations.AlterField(
            model_name="artifactethnicity",
            name="feature_ref_is_name",
            field=lamindb.base.fields.BooleanField(blank=True, default=None, null=True),
        ),
        migrations.AlterField(
            model_name="artifactethnicity",
            name="label_ref_is_name",
            field=lamindb.base.fields.BooleanField(blank=True, default=None, null=True),
        ),
        migrations.AlterField(
            model_name="artifactethnicity",
            name="run",
            field=lamindb.base.fields.ForeignKey(
                blank=True,
                default=lamindb.models.current_run,
                null=True,
                on_delete=django.db.models.deletion.PROTECT,
                related_name="+",
                to="lamindb.run",
            ),
        ),
        migrations.AlterField(
            model_name="artifactexperimentalfactor",
            name="artifact",
            field=lamindb.base.fields.ForeignKey(
                blank=True,
                on_delete=django.db.models.deletion.CASCADE,
                related_name="links_experimental_factor",
                to="lamindb.artifact",
            ),
        ),
        migrations.AlterField(
            model_name="artifactexperimentalfactor",
            name="created_at",
            field=lamindb.base.fields.DateTimeField(auto_now_add=True, db_index=True),
        ),
        migrations.AlterField(
            model_name="artifactexperimentalfactor",
            name="created_by",
            field=lamindb.base.fields.ForeignKey(
                blank=True,
                default=lamindb.base.users.current_user_id,
                on_delete=django.db.models.deletion.PROTECT,
                related_name="+",
                to="lamindb.user",
            ),
        ),
        migrations.AlterField(
            model_name="artifactexperimentalfactor",
            name="experimentalfactor",
            field=lamindb.base.fields.ForeignKey(
                blank=True,
                on_delete=django.db.models.deletion.PROTECT,
                related_name="links_artifact",
                to="bionty.experimentalfactor",
            ),
        ),
        migrations.AlterField(
            model_name="artifactexperimentalfactor",
            name="feature",
            field=lamindb.base.fields.ForeignKey(
                blank=True,
                default=None,
                null=True,
                on_delete=django.db.models.deletion.PROTECT,
                related_name="links_artifactexperimentalfactor",
                to="lamindb.feature",
            ),
        ),
        migrations.AlterField(
            model_name="artifactexperimentalfactor",
            name="feature_ref_is_name",
            field=lamindb.base.fields.BooleanField(blank=True, default=None, null=True),
        ),
        migrations.AlterField(
            model_name="artifactexperimentalfactor",
            name="label_ref_is_name",
            field=lamindb.base.fields.BooleanField(blank=True, default=None, null=True),
        ),
        migrations.AlterField(
            model_name="artifactexperimentalfactor",
            name="run",
            field=lamindb.base.fields.ForeignKey(
                blank=True,
                default=lamindb.models.current_run,
                null=True,
                on_delete=django.db.models.deletion.PROTECT,
                related_name="+",
                to="lamindb.run",
            ),
        ),
        migrations.AlterField(
            model_name="artifactgene",
            name="artifact",
            field=lamindb.base.fields.ForeignKey(
                blank=True,
                on_delete=django.db.models.deletion.CASCADE,
                related_name="links_gene",
                to="lamindb.artifact",
            ),
        ),
        migrations.AlterField(
            model_name="artifactgene",
            name="created_at",
            field=lamindb.base.fields.DateTimeField(auto_now_add=True, db_index=True),
        ),
        migrations.AlterField(
            model_name="artifactgene",
            name="created_by",
            field=lamindb.base.fields.ForeignKey(
                blank=True,
                default=lamindb.base.users.current_user_id,
                on_delete=django.db.models.deletion.PROTECT,
                related_name="+",
                to="lamindb.user",
            ),
        ),
        migrations.AlterField(
            model_name="artifactgene",
            name="feature",
            field=lamindb.base.fields.ForeignKey(
                blank=True,
                default=None,
                null=True,
                on_delete=django.db.models.deletion.PROTECT,
                related_name="links_artifactgene",
                to="lamindb.feature",
            ),
        ),
        migrations.AlterField(
            model_name="artifactgene",
            name="feature_ref_is_name",
            field=lamindb.base.fields.BooleanField(blank=True, default=None, null=True),
        ),
        migrations.AlterField(
            model_name="artifactgene",
            name="gene",
            field=lamindb.base.fields.ForeignKey(
                blank=True,
                on_delete=django.db.models.deletion.PROTECT,
                related_name="links_artifact",
                to="bionty.gene",
            ),
        ),
        migrations.AlterField(
            model_name="artifactgene",
            name="label_ref_is_name",
            field=lamindb.base.fields.BooleanField(blank=True, default=None, null=True),
        ),
        migrations.AlterField(
            model_name="artifactgene",
            name="run",
            field=lamindb.base.fields.ForeignKey(
                blank=True,
                default=lamindb.models.current_run,
                null=True,
                on_delete=django.db.models.deletion.PROTECT,
                related_name="+",
                to="lamindb.run",
            ),
        ),
        migrations.AlterField(
            model_name="artifactorganism",
            name="artifact",
            field=lamindb.base.fields.ForeignKey(
                blank=True,
                on_delete=django.db.models.deletion.CASCADE,
                related_name="links_organism",
                to="lamindb.artifact",
            ),
        ),
        migrations.AlterField(
            model_name="artifactorganism",
            name="created_at",
            field=lamindb.base.fields.DateTimeField(auto_now_add=True, db_index=True),
        ),
        migrations.AlterField(
            model_name="artifactorganism",
            name="created_by",
            field=lamindb.base.fields.ForeignKey(
                blank=True,
                default=lamindb.base.users.current_user_id,
                on_delete=django.db.models.deletion.PROTECT,
                related_name="+",
                to="lamindb.user",
            ),
        ),
        migrations.AlterField(
            model_name="artifactorganism",
            name="feature",
            field=lamindb.base.fields.ForeignKey(
                blank=True,
                default=None,
                null=True,
                on_delete=django.db.models.deletion.PROTECT,
                related_name="links_artifactorganism",
                to="lamindb.feature",
            ),
        ),
        migrations.AlterField(
            model_name="artifactorganism",
            name="feature_ref_is_name",
            field=lamindb.base.fields.BooleanField(blank=True, default=None, null=True),
        ),
        migrations.AlterField(
            model_name="artifactorganism",
            name="label_ref_is_name",
            field=lamindb.base.fields.BooleanField(blank=True, default=None, null=True),
        ),
        migrations.AlterField(
            model_name="artifactorganism",
            name="organism",
            field=lamindb.base.fields.ForeignKey(
                blank=True,
                on_delete=django.db.models.deletion.PROTECT,
                related_name="links_artifact",
                to="bionty.organism",
            ),
        ),
        migrations.AlterField(
            model_name="artifactorganism",
            name="run",
            field=lamindb.base.fields.ForeignKey(
                blank=True,
                default=lamindb.models.current_run,
                null=True,
                on_delete=django.db.models.deletion.PROTECT,
                related_name="+",
                to="lamindb.run",
            ),
        ),
        migrations.AlterField(
            model_name="artifactpathway",
            name="artifact",
            field=lamindb.base.fields.ForeignKey(
                blank=True,
                on_delete=django.db.models.deletion.CASCADE,
                related_name="links_pathway",
                to="lamindb.artifact",
            ),
        ),
        migrations.AlterField(
            model_name="artifactpathway",
            name="created_at",
            field=lamindb.base.fields.DateTimeField(auto_now_add=True, db_index=True),
        ),
        migrations.AlterField(
            model_name="artifactpathway",
            name="created_by",
            field=lamindb.base.fields.ForeignKey(
                blank=True,
                default=lamindb.base.users.current_user_id,
                on_delete=django.db.models.deletion.PROTECT,
                related_name="+",
                to="lamindb.user",
            ),
        ),
        migrations.AlterField(
            model_name="artifactpathway",
            name="feature",
            field=lamindb.base.fields.ForeignKey(
                blank=True,
                default=None,
                null=True,
                on_delete=django.db.models.deletion.PROTECT,
                related_name="links_artifactpathway",
                to="lamindb.feature",
            ),
        ),
        migrations.AlterField(
            model_name="artifactpathway",
            name="feature_ref_is_name",
            field=lamindb.base.fields.BooleanField(blank=True, default=None, null=True),
        ),
        migrations.AlterField(
            model_name="artifactpathway",
            name="label_ref_is_name",
            field=lamindb.base.fields.BooleanField(blank=True, default=None, null=True),
        ),
        migrations.AlterField(
            model_name="artifactpathway",
            name="pathway",
            field=lamindb.base.fields.ForeignKey(
                blank=True,
                on_delete=django.db.models.deletion.PROTECT,
                related_name="links_artifact",
                to="bionty.pathway",
            ),
        ),
        migrations.AlterField(
            model_name="artifactpathway",
            name="run",
            field=lamindb.base.fields.ForeignKey(
                blank=True,
                default=lamindb.models.current_run,
                null=True,
                on_delete=django.db.models.deletion.PROTECT,
                related_name="+",
                to="lamindb.run",
            ),
        ),
        migrations.AlterField(
            model_name="artifactphenotype",
            name="artifact",
            field=lamindb.base.fields.ForeignKey(
                blank=True,
                on_delete=django.db.models.deletion.CASCADE,
                related_name="links_phenotype",
                to="lamindb.artifact",
            ),
        ),
        migrations.AlterField(
            model_name="artifactphenotype",
            name="created_at",
            field=lamindb.base.fields.DateTimeField(auto_now_add=True, db_index=True),
        ),
        migrations.AlterField(
            model_name="artifactphenotype",
            name="created_by",
            field=lamindb.base.fields.ForeignKey(
                blank=True,
                default=lamindb.base.users.current_user_id,
                on_delete=django.db.models.deletion.PROTECT,
                related_name="+",
                to="lamindb.user",
            ),
        ),
        migrations.AlterField(
            model_name="artifactphenotype",
            name="feature",
            field=lamindb.base.fields.ForeignKey(
                blank=True,
                default=None,
                null=True,
                on_delete=django.db.models.deletion.PROTECT,
                related_name="links_artifactphenotype",
                to="lamindb.feature",
            ),
        ),
        migrations.AlterField(
            model_name="artifactphenotype",
            name="feature_ref_is_name",
            field=lamindb.base.fields.BooleanField(blank=True, default=None, null=True),
        ),
        migrations.AlterField(
            model_name="artifactphenotype",
            name="label_ref_is_name",
            field=lamindb.base.fields.BooleanField(blank=True, default=None, null=True),
        ),
        migrations.AlterField(
            model_name="artifactphenotype",
            name="phenotype",
            field=lamindb.base.fields.ForeignKey(
                blank=True,
                on_delete=django.db.models.deletion.PROTECT,
                related_name="links_artifact",
                to="bionty.phenotype",
            ),
        ),
        migrations.AlterField(
            model_name="artifactphenotype",
            name="run",
            field=lamindb.base.fields.ForeignKey(
                blank=True,
                default=lamindb.models.current_run,
                null=True,
                on_delete=django.db.models.deletion.PROTECT,
                related_name="+",
                to="lamindb.run",
            ),
        ),
        migrations.AlterField(
            model_name="artifactprotein",
            name="artifact",
            field=lamindb.base.fields.ForeignKey(
                blank=True,
                on_delete=django.db.models.deletion.CASCADE,
                related_name="links_protein",
                to="lamindb.artifact",
            ),
        ),
        migrations.AlterField(
            model_name="artifactprotein",
            name="created_at",
            field=lamindb.base.fields.DateTimeField(auto_now_add=True, db_index=True),
        ),
        migrations.AlterField(
            model_name="artifactprotein",
            name="created_by",
            field=lamindb.base.fields.ForeignKey(
                blank=True,
                default=lamindb.base.users.current_user_id,
                on_delete=django.db.models.deletion.PROTECT,
                related_name="+",
                to="lamindb.user",
            ),
        ),
        migrations.AlterField(
            model_name="artifactprotein",
            name="feature",
            field=lamindb.base.fields.ForeignKey(
                blank=True,
                default=None,
                null=True,
                on_delete=django.db.models.deletion.PROTECT,
                related_name="links_artifactprotein",
                to="lamindb.feature",
            ),
        ),
        migrations.AlterField(
            model_name="artifactprotein",
            name="feature_ref_is_name",
            field=lamindb.base.fields.BooleanField(blank=True, default=None, null=True),
        ),
        migrations.AlterField(
            model_name="artifactprotein",
            name="label_ref_is_name",
            field=lamindb.base.fields.BooleanField(blank=True, default=None, null=True),
        ),
        migrations.AlterField(
            model_name="artifactprotein",
            name="protein",
            field=lamindb.base.fields.ForeignKey(
                blank=True,
                on_delete=django.db.models.deletion.PROTECT,
                related_name="links_artifact",
                to="bionty.protein",
            ),
        ),
        migrations.AlterField(
            model_name="artifactprotein",
            name="run",
            field=lamindb.base.fields.ForeignKey(
                blank=True,
                default=lamindb.models.current_run,
                null=True,
                on_delete=django.db.models.deletion.PROTECT,
                related_name="+",
                to="lamindb.run",
            ),
        ),
        migrations.AlterField(
            model_name="artifacttissue",
            name="artifact",
            field=lamindb.base.fields.ForeignKey(
                blank=True,
                on_delete=django.db.models.deletion.CASCADE,
                related_name="links_tissue",
                to="lamindb.artifact",
            ),
        ),
        migrations.AlterField(
            model_name="artifacttissue",
            name="created_at",
            field=lamindb.base.fields.DateTimeField(auto_now_add=True, db_index=True),
        ),
        migrations.AlterField(
            model_name="artifacttissue",
            name="created_by",
            field=lamindb.base.fields.ForeignKey(
                blank=True,
                default=lamindb.base.users.current_user_id,
                on_delete=django.db.models.deletion.PROTECT,
                related_name="+",
                to="lamindb.user",
            ),
        ),
        migrations.AlterField(
            model_name="artifacttissue",
            name="feature",
            field=lamindb.base.fields.ForeignKey(
                blank=True,
                default=None,
                null=True,
                on_delete=django.db.models.deletion.PROTECT,
                related_name="links_artifacttissue",
                to="lamindb.feature",
            ),
        ),
        migrations.AlterField(
            model_name="artifacttissue",
            name="feature_ref_is_name",
            field=lamindb.base.fields.BooleanField(blank=True, default=None, null=True),
        ),
        migrations.AlterField(
            model_name="artifacttissue",
            name="label_ref_is_name",
            field=lamindb.base.fields.BooleanField(blank=True, default=None, null=True),
        ),
        migrations.AlterField(
            model_name="artifacttissue",
            name="run",
            field=lamindb.base.fields.ForeignKey(
                blank=True,
                default=lamindb.models.current_run,
                null=True,
                on_delete=django.db.models.deletion.PROTECT,
                related_name="+",
                to="lamindb.run",
            ),
        ),
        migrations.AlterField(
            model_name="artifacttissue",
            name="tissue",
            field=lamindb.base.fields.ForeignKey(
                blank=True,
                on_delete=django.db.models.deletion.PROTECT,
                related_name="links_artifact",
                to="bionty.tissue",
            ),
        ),
        migrations.AlterField(
            model_name="cellline",
            name="abbr",
            field=lamindb.base.fields.CharField(
                blank=True,
                db_index=True,
                default=None,
                max_length=32,
                null=True,
                unique=True,
            ),
        ),
        migrations.AlterField(
            model_name="cellline",
            name="created_at",
            field=lamindb.base.fields.DateTimeField(auto_now_add=True, db_index=True),
        ),
        migrations.AlterField(
            model_name="cellline",
            name="created_by",
            field=lamindb.base.fields.ForeignKey(
                blank=True,
                default=lamindb.base.users.current_user_id,
                on_delete=django.db.models.deletion.PROTECT,
                related_name="+",
                to="lamindb.user",
            ),
        ),
        migrations.AlterField(
            model_name="cellline",
            name="description",
            field=lamindb.base.fields.TextField(blank=True, default=None, null=True),
        ),
        migrations.AlterField(
            model_name="cellline",
            name="name",
            field=lamindb.base.fields.CharField(
                blank=True, db_index=True, default=None, max_length=256
            ),
        ),
        migrations.AlterField(
            model_name="cellline",
            name="ontology_id",
            field=lamindb.base.fields.CharField(
                blank=True, db_index=True, default=None, max_length=32, null=True
            ),
        ),
        migrations.AlterField(
            model_name="cellline",
            name="run",
            field=lamindb.base.fields.ForeignKey(
                blank=True,
                default=lamindb.models.current_run,
                null=True,
                on_delete=django.db.models.deletion.PROTECT,
                related_name="+",
                to="lamindb.run",
            ),
        ),
        migrations.AlterField(
            model_name="cellline",
            name="source",
            field=lamindb.base.fields.ForeignKey(
                blank=True,
                null=True,
                on_delete=django.db.models.deletion.PROTECT,
                related_name="+",
                to="bionty.source",
            ),
        ),
        migrations.AlterField(
            model_name="cellline",
            name="synonyms",
            field=lamindb.base.fields.TextField(blank=True, default=None, null=True),
        ),
        migrations.AlterField(
            model_name="cellline",
            name="uid",
            field=lamindb.base.fields.CharField(
                blank=True, default=bionty.ids.ontology, max_length=8, unique=True
            ),
        ),
        migrations.AlterField(
            model_name="cellline",
            name="updated_at",
            field=lamindb.base.fields.DateTimeField(auto_now=True, db_index=True),
        ),
        migrations.AlterField(
            model_name="cellmarker",
            name="created_at",
            field=lamindb.base.fields.DateTimeField(auto_now_add=True, db_index=True),
        ),
        migrations.AlterField(
            model_name="cellmarker",
            name="created_by",
            field=lamindb.base.fields.ForeignKey(
                blank=True,
                default=lamindb.base.users.current_user_id,
                on_delete=django.db.models.deletion.PROTECT,
                related_name="+",
                to="lamindb.user",
            ),
        ),
        migrations.AlterField(
            model_name="cellmarker",
            name="description",
            field=lamindb.base.fields.TextField(blank=True, default=None, null=True),
        ),
        migrations.AlterField(
            model_name="cellmarker",
            name="gene_symbol",
            field=lamindb.base.fields.CharField(
                blank=True, db_index=True, default=None, max_length=64, null=True
            ),
        ),
        migrations.AlterField(
            model_name="cellmarker",
            name="name",
            field=lamindb.base.fields.CharField(
                blank=True, db_index=True, default=None, max_length=64
            ),
        ),
        migrations.AlterField(
            model_name="cellmarker",
            name="ncbi_gene_id",
            field=lamindb.base.fields.CharField(
                blank=True, db_index=True, default=None, max_length=32, null=True
            ),
        ),
        migrations.AlterField(
            model_name="cellmarker",
            name="organism",
            field=lamindb.base.fields.ForeignKey(
                blank=True,
                default=None,
                on_delete=django.db.models.deletion.PROTECT,
                related_name="cell_markers",
                to="bionty.organism",
            ),
        ),
        migrations.AlterField(
            model_name="cellmarker",
            name="run",
            field=lamindb.base.fields.ForeignKey(
                blank=True,
                default=lamindb.models.current_run,
                null=True,
                on_delete=django.db.models.deletion.PROTECT,
                related_name="+",
                to="lamindb.run",
            ),
        ),
        migrations.AlterField(
            model_name="cellmarker",
            name="source",
            field=lamindb.base.fields.ForeignKey(
                blank=True,
                null=True,
                on_delete=django.db.models.deletion.PROTECT,
                related_name="+",
                to="bionty.source",
            ),
        ),
        migrations.AlterField(
            model_name="cellmarker",
            name="synonyms",
            field=lamindb.base.fields.TextField(blank=True, default=None, null=True),
        ),
        migrations.AlterField(
            model_name="cellmarker",
            name="uid",
            field=lamindb.base.fields.CharField(
                blank=True, default=bionty.ids.cellmarker, max_length=12, unique=True
            ),
        ),
        migrations.AlterField(
            model_name="cellmarker",
            name="uniprotkb_id",
            field=lamindb.base.fields.CharField(
                blank=True, db_index=True, default=None, max_length=10, null=True
            ),
        ),
        migrations.AlterField(
            model_name="cellmarker",
            name="updated_at",
            field=lamindb.base.fields.DateTimeField(auto_now=True, db_index=True),
        ),
        migrations.AlterField(
            model_name="celltype",
            name="abbr",
            field=lamindb.base.fields.CharField(
                blank=True,
                db_index=True,
                default=None,
                max_length=32,
                null=True,
                unique=True,
            ),
        ),
        migrations.AlterField(
            model_name="celltype",
            name="created_at",
            field=lamindb.base.fields.DateTimeField(auto_now_add=True, db_index=True),
        ),
        migrations.AlterField(
            model_name="celltype",
            name="created_by",
            field=lamindb.base.fields.ForeignKey(
                blank=True,
                default=lamindb.base.users.current_user_id,
                on_delete=django.db.models.deletion.PROTECT,
                related_name="+",
                to="lamindb.user",
            ),
        ),
        migrations.AlterField(
            model_name="celltype",
            name="description",
            field=lamindb.base.fields.TextField(blank=True, default=None, null=True),
        ),
        migrations.AlterField(
            model_name="celltype",
            name="name",
            field=lamindb.base.fields.CharField(
                blank=True, db_index=True, default=None, max_length=256
            ),
        ),
        migrations.AlterField(
            model_name="celltype",
            name="ontology_id",
            field=lamindb.base.fields.CharField(
                blank=True, db_index=True, default=None, max_length=32, null=True
            ),
        ),
        migrations.AlterField(
            model_name="celltype",
            name="run",
            field=lamindb.base.fields.ForeignKey(
                blank=True,
                default=lamindb.models.current_run,
                null=True,
                on_delete=django.db.models.deletion.PROTECT,
                related_name="+",
                to="lamindb.run",
            ),
        ),
        migrations.AlterField(
            model_name="celltype",
            name="source",
            field=lamindb.base.fields.ForeignKey(
                blank=True,
                null=True,
                on_delete=django.db.models.deletion.PROTECT,
                related_name="+",
                to="bionty.source",
            ),
        ),
        migrations.AlterField(
            model_name="celltype",
            name="synonyms",
            field=lamindb.base.fields.TextField(blank=True, default=None, null=True),
        ),
        migrations.AlterField(
            model_name="celltype",
            name="uid",
            field=lamindb.base.fields.CharField(
                blank=True, default=bionty.ids.ontology, max_length=8, unique=True
            ),
        ),
        migrations.AlterField(
            model_name="celltype",
            name="updated_at",
            field=lamindb.base.fields.DateTimeField(auto_now=True, db_index=True),
        ),
        migrations.AlterField(
            model_name="developmentalstage",
            name="abbr",
            field=lamindb.base.fields.CharField(
                blank=True,
                db_index=True,
                default=None,
                max_length=32,
                null=True,
                unique=True,
            ),
        ),
        migrations.AlterField(
            model_name="developmentalstage",
            name="created_at",
            field=lamindb.base.fields.DateTimeField(auto_now_add=True, db_index=True),
        ),
        migrations.AlterField(
            model_name="developmentalstage",
            name="created_by",
            field=lamindb.base.fields.ForeignKey(
                blank=True,
                default=lamindb.base.users.current_user_id,
                on_delete=django.db.models.deletion.PROTECT,
                related_name="+",
                to="lamindb.user",
            ),
        ),
        migrations.AlterField(
            model_name="developmentalstage",
            name="description",
            field=lamindb.base.fields.TextField(blank=True, default=None, null=True),
        ),
        migrations.AlterField(
            model_name="developmentalstage",
            name="name",
            field=lamindb.base.fields.CharField(
                blank=True, db_index=True, default=None, max_length=256
            ),
        ),
        migrations.AlterField(
            model_name="developmentalstage",
            name="ontology_id",
            field=lamindb.base.fields.CharField(
                blank=True, db_index=True, default=None, max_length=32, null=True
            ),
        ),
        migrations.AlterField(
            model_name="developmentalstage",
            name="run",
            field=lamindb.base.fields.ForeignKey(
                blank=True,
                default=lamindb.models.current_run,
                null=True,
                on_delete=django.db.models.deletion.PROTECT,
                related_name="+",
                to="lamindb.run",
            ),
        ),
        migrations.AlterField(
            model_name="developmentalstage",
            name="source",
            field=lamindb.base.fields.ForeignKey(
                blank=True,
                null=True,
                on_delete=django.db.models.deletion.PROTECT,
                related_name="+",
                to="bionty.source",
            ),
        ),
        migrations.AlterField(
            model_name="developmentalstage",
            name="synonyms",
            field=lamindb.base.fields.TextField(blank=True, default=None, null=True),
        ),
        migrations.AlterField(
            model_name="developmentalstage",
            name="uid",
            field=lamindb.base.fields.CharField(
                blank=True, default=bionty.ids.ontology, max_length=8, unique=True
            ),
        ),
        migrations.AlterField(
            model_name="developmentalstage",
            name="updated_at",
            field=lamindb.base.fields.DateTimeField(auto_now=True, db_index=True),
        ),
        migrations.AlterField(
            model_name="disease",
            name="abbr",
            field=lamindb.base.fields.CharField(
                blank=True,
                db_index=True,
                default=None,
                max_length=32,
                null=True,
                unique=True,
            ),
        ),
        migrations.AlterField(
            model_name="disease",
            name="created_at",
            field=lamindb.base.fields.DateTimeField(auto_now_add=True, db_index=True),
        ),
        migrations.AlterField(
            model_name="disease",
            name="created_by",
            field=lamindb.base.fields.ForeignKey(
                blank=True,
                default=lamindb.base.users.current_user_id,
                on_delete=django.db.models.deletion.PROTECT,
                related_name="+",
                to="lamindb.user",
            ),
        ),
        migrations.AlterField(
            model_name="disease",
            name="description",
            field=lamindb.base.fields.TextField(blank=True, default=None, null=True),
        ),
        migrations.AlterField(
            model_name="disease",
            name="name",
            field=lamindb.base.fields.CharField(
                blank=True, db_index=True, default=None, max_length=256
            ),
        ),
        migrations.AlterField(
            model_name="disease",
            name="ontology_id",
            field=lamindb.base.fields.CharField(
                blank=True, db_index=True, default=None, max_length=32, null=True
            ),
        ),
        migrations.AlterField(
            model_name="disease",
            name="run",
            field=lamindb.base.fields.ForeignKey(
                blank=True,
                default=lamindb.models.current_run,
                null=True,
                on_delete=django.db.models.deletion.PROTECT,
                related_name="+",
                to="lamindb.run",
            ),
        ),
        migrations.AlterField(
            model_name="disease",
            name="source",
            field=lamindb.base.fields.ForeignKey(
                blank=True,
                null=True,
                on_delete=django.db.models.deletion.PROTECT,
                related_name="+",
                to="bionty.source",
            ),
        ),
        migrations.AlterField(
            model_name="disease",
            name="synonyms",
            field=lamindb.base.fields.TextField(blank=True, default=None, null=True),
        ),
        migrations.AlterField(
            model_name="disease",
            name="uid",
            field=lamindb.base.fields.CharField(
                blank=True, default=bionty.ids.ontology, max_length=8, unique=True
            ),
        ),
        migrations.AlterField(
            model_name="disease",
            name="updated_at",
            field=lamindb.base.fields.DateTimeField(auto_now=True, db_index=True),
        ),
        migrations.AlterField(
            model_name="ethnicity",
            name="abbr",
            field=lamindb.base.fields.CharField(
                blank=True,
                db_index=True,
                default=None,
                max_length=32,
                null=True,
                unique=True,
            ),
        ),
        migrations.AlterField(
            model_name="ethnicity",
            name="created_at",
            field=lamindb.base.fields.DateTimeField(auto_now_add=True, db_index=True),
        ),
        migrations.AlterField(
            model_name="ethnicity",
            name="created_by",
            field=lamindb.base.fields.ForeignKey(
                blank=True,
                default=lamindb.base.users.current_user_id,
                on_delete=django.db.models.deletion.PROTECT,
                related_name="+",
                to="lamindb.user",
            ),
        ),
        migrations.AlterField(
            model_name="ethnicity",
            name="description",
            field=lamindb.base.fields.TextField(blank=True, default=None, null=True),
        ),
        migrations.AlterField(
            model_name="ethnicity",
            name="name",
            field=lamindb.base.fields.CharField(
                blank=True, db_index=True, default=None, max_length=256
            ),
        ),
        migrations.AlterField(
            model_name="ethnicity",
            name="ontology_id",
            field=lamindb.base.fields.CharField(
                blank=True, db_index=True, default=None, max_length=32, null=True
            ),
        ),
        migrations.AlterField(
            model_name="ethnicity",
            name="run",
            field=lamindb.base.fields.ForeignKey(
                blank=True,
                default=lamindb.models.current_run,
                null=True,
                on_delete=django.db.models.deletion.PROTECT,
                related_name="+",
                to="lamindb.run",
            ),
        ),
        migrations.AlterField(
            model_name="ethnicity",
            name="source",
            field=lamindb.base.fields.ForeignKey(
                blank=True,
                null=True,
                on_delete=django.db.models.deletion.PROTECT,
                related_name="+",
                to="bionty.source",
            ),
        ),
        migrations.AlterField(
            model_name="ethnicity",
            name="synonyms",
            field=lamindb.base.fields.TextField(blank=True, default=None, null=True),
        ),
        migrations.AlterField(
            model_name="ethnicity",
            name="uid",
            field=lamindb.base.fields.CharField(
                blank=True, default=bionty.ids.ontology, max_length=8, unique=True
            ),
        ),
        migrations.AlterField(
            model_name="ethnicity",
            name="updated_at",
            field=lamindb.base.fields.DateTimeField(auto_now=True, db_index=True),
        ),
        migrations.AlterField(
            model_name="experimentalfactor",
            name="abbr",
            field=lamindb.base.fields.CharField(
                blank=True,
                db_index=True,
                default=None,
                max_length=32,
                null=True,
                unique=True,
            ),
        ),
        migrations.AlterField(
            model_name="experimentalfactor",
            name="created_at",
            field=lamindb.base.fields.DateTimeField(auto_now_add=True, db_index=True),
        ),
        migrations.AlterField(
            model_name="experimentalfactor",
            name="created_by",
            field=lamindb.base.fields.ForeignKey(
                blank=True,
                default=lamindb.base.users.current_user_id,
                on_delete=django.db.models.deletion.PROTECT,
                related_name="+",
                to="lamindb.user",
            ),
        ),
        migrations.AlterField(
            model_name="experimentalfactor",
            name="description",
            field=lamindb.base.fields.TextField(blank=True, default=None, null=True),
        ),
        migrations.AlterField(
            model_name="experimentalfactor",
            name="instrument",
            field=lamindb.base.fields.TextField(
                blank=True, db_index=True, default=None, null=True
            ),
        ),
        migrations.AlterField(
            model_name="experimentalfactor",
            name="measurement",
            field=lamindb.base.fields.TextField(
                blank=True, db_index=True, default=None, null=True
            ),
        ),
        migrations.AlterField(
            model_name="experimentalfactor",
            name="molecule",
            field=lamindb.base.fields.TextField(
                blank=True, db_index=True, default=None, null=True
            ),
        ),
        migrations.AlterField(
            model_name="experimentalfactor",
            name="name",
            field=lamindb.base.fields.CharField(
                blank=True, db_index=True, default=None, max_length=256
            ),
        ),
        migrations.AlterField(
            model_name="experimentalfactor",
            name="ontology_id",
            field=lamindb.base.fields.CharField(
                blank=True, db_index=True, default=None, max_length=32, null=True
            ),
        ),
        migrations.AlterField(
            model_name="experimentalfactor",
            name="run",
            field=lamindb.base.fields.ForeignKey(
                blank=True,
                default=lamindb.models.current_run,
                null=True,
                on_delete=django.db.models.deletion.PROTECT,
                related_name="+",
                to="lamindb.run",
            ),
        ),
        migrations.AlterField(
            model_name="experimentalfactor",
            name="source",
            field=lamindb.base.fields.ForeignKey(
                blank=True,
                null=True,
                on_delete=django.db.models.deletion.PROTECT,
                related_name="+",
                to="bionty.source",
            ),
        ),
        migrations.AlterField(
            model_name="experimentalfactor",
            name="synonyms",
            field=lamindb.base.fields.TextField(blank=True, default=None, null=True),
        ),
        migrations.AlterField(
            model_name="experimentalfactor",
            name="uid",
            field=lamindb.base.fields.CharField(
                blank=True, default=bionty.ids.ontology, max_length=8, unique=True
            ),
        ),
        migrations.AlterField(
            model_name="experimentalfactor",
            name="updated_at",
            field=lamindb.base.fields.DateTimeField(auto_now=True, db_index=True),
        ),
        migrations.AlterField(
            model_name="featuresetcellmarker",
            name="cellmarker",
            field=lamindb.base.fields.ForeignKey(
                blank=True,
                on_delete=django.db.models.deletion.PROTECT,
                related_name="+",
                to="bionty.cellmarker",
            ),
        ),
        migrations.AlterField(
            model_name="featuresetcellmarker",
            name="featureset",
            field=lamindb.base.fields.ForeignKey(
                blank=True,
                on_delete=django.db.models.deletion.CASCADE,
                related_name="+",
                to="lamindb.featureset",
            ),
        ),
        migrations.AlterField(
            model_name="featuresetgene",
            name="featureset",
            field=lamindb.base.fields.ForeignKey(
                blank=True,
                on_delete=django.db.models.deletion.CASCADE,
                related_name="+",
                to="lamindb.featureset",
            ),
        ),
        migrations.AlterField(
            model_name="featuresetgene",
            name="gene",
            field=lamindb.base.fields.ForeignKey(
                blank=True,
                on_delete=django.db.models.deletion.PROTECT,
                related_name="+",
                to="bionty.gene",
            ),
        ),
        migrations.AlterField(
            model_name="featuresetpathway",
            name="featureset",
            field=lamindb.base.fields.ForeignKey(
                blank=True,
                on_delete=django.db.models.deletion.CASCADE,
                related_name="+",
                to="lamindb.featureset",
            ),
        ),
        migrations.AlterField(
            model_name="featuresetpathway",
            name="pathway",
            field=lamindb.base.fields.ForeignKey(
                blank=True,
                on_delete=django.db.models.deletion.PROTECT,
                related_name="+",
                to="bionty.pathway",
            ),
        ),
        migrations.AlterField(
            model_name="featuresetprotein",
            name="featureset",
            field=lamindb.base.fields.ForeignKey(
                blank=True,
                on_delete=django.db.models.deletion.CASCADE,
                related_name="+",
                to="lamindb.featureset",
            ),
        ),
        migrations.AlterField(
            model_name="featuresetprotein",
            name="protein",
            field=lamindb.base.fields.ForeignKey(
                blank=True,
                on_delete=django.db.models.deletion.PROTECT,
                related_name="+",
                to="bionty.protein",
            ),
        ),
        migrations.AlterField(
            model_name="gene",
            name="biotype",
            field=lamindb.base.fields.CharField(
                blank=True, db_index=True, default=None, max_length=64, null=True
            ),
        ),
        migrations.AlterField(
            model_name="gene",
            name="created_at",
            field=lamindb.base.fields.DateTimeField(auto_now_add=True, db_index=True),
        ),
        migrations.AlterField(
            model_name="gene",
            name="created_by",
            field=lamindb.base.fields.ForeignKey(
                blank=True,
                default=lamindb.base.users.current_user_id,
                on_delete=django.db.models.deletion.PROTECT,
                related_name="+",
                to="lamindb.user",
            ),
        ),
        migrations.AlterField(
            model_name="gene",
            name="description",
            field=lamindb.base.fields.TextField(blank=True, default=None, null=True),
        ),
        migrations.AlterField(
            model_name="gene",
            name="ensembl_gene_id",
            field=lamindb.base.fields.CharField(
                blank=True,
                db_index=True,
                default=None,
                max_length=64,
                null=True,
                unique=True,
            ),
        ),
        migrations.AlterField(
            model_name="gene",
            name="ncbi_gene_ids",
            field=lamindb.base.fields.TextField(blank=True, default=None, null=True),
        ),
        migrations.AlterField(
            model_name="gene",
            name="organism",
            field=lamindb.base.fields.ForeignKey(
                blank=True,
                default=None,
                on_delete=django.db.models.deletion.PROTECT,
                related_name="genes",
                to="bionty.organism",
            ),
        ),
        migrations.AlterField(
            model_name="gene",
            name="run",
            field=lamindb.base.fields.ForeignKey(
                blank=True,
                default=lamindb.models.current_run,
                null=True,
                on_delete=django.db.models.deletion.PROTECT,
                related_name="+",
                to="lamindb.run",
            ),
        ),
        migrations.AlterField(
            model_name="gene",
            name="source",
            field=lamindb.base.fields.ForeignKey(
                blank=True,
                null=True,
                on_delete=django.db.models.deletion.PROTECT,
                related_name="+",
                to="bionty.source",
            ),
        ),
        migrations.AlterField(
            model_name="gene",
            name="stable_id",
            field=lamindb.base.fields.CharField(
                blank=True,
                db_index=True,
                default=None,
                max_length=64,
                null=True,
                unique=True,
            ),
        ),
        migrations.AlterField(
            model_name="gene",
            name="symbol",
            field=lamindb.base.fields.CharField(
                blank=True, db_index=True, default=None, max_length=64, null=True
            ),
        ),
        migrations.AlterField(
            model_name="gene",
            name="synonyms",
            field=lamindb.base.fields.TextField(blank=True, default=None, null=True),
        ),
        migrations.AlterField(
            model_name="gene",
            name="uid",
            field=lamindb.base.fields.CharField(
                blank=True, default=bionty.ids.gene, max_length=12, unique=True
            ),
        ),
        migrations.AlterField(
            model_name="gene",
            name="updated_at",
            field=lamindb.base.fields.DateTimeField(auto_now=True, db_index=True),
        ),
        migrations.AlterField(
            model_name="organism",
            name="created_at",
            field=lamindb.base.fields.DateTimeField(auto_now_add=True, db_index=True),
        ),
        migrations.AlterField(
            model_name="organism",
            name="created_by",
            field=lamindb.base.fields.ForeignKey(
                blank=True,
                default=lamindb.base.users.current_user_id,
                on_delete=django.db.models.deletion.PROTECT,
                related_name="+",
                to="lamindb.user",
            ),
        ),
        migrations.AlterField(
            model_name="organism",
            name="description",
            field=lamindb.base.fields.TextField(blank=True, default=None, null=True),
        ),
        migrations.AlterField(
            model_name="organism",
            name="name",
            field=lamindb.base.fields.CharField(
                blank=True, db_index=True, default=None, max_length=64, unique=True
            ),
        ),
        migrations.AlterField(
            model_name="organism",
            name="ontology_id",
            field=lamindb.base.fields.CharField(
                blank=True,
                db_index=True,
                default=None,
                max_length=32,
                null=True,
                unique=True,
            ),
        ),
        migrations.AlterField(
            model_name="organism",
            name="run",
            field=lamindb.base.fields.ForeignKey(
                blank=True,
                default=lamindb.models.current_run,
                null=True,
                on_delete=django.db.models.deletion.PROTECT,
                related_name="+",
                to="lamindb.run",
            ),
        ),
        migrations.AlterField(
            model_name="organism",
            name="scientific_name",
            field=lamindb.base.fields.CharField(
                blank=True,
                db_index=True,
                default=None,
                max_length=64,
                null=True,
                unique=True,
            ),
        ),
        migrations.AlterField(
            model_name="organism",
            name="source",
            field=lamindb.base.fields.ForeignKey(
                blank=True,
                null=True,
                on_delete=django.db.models.deletion.PROTECT,
                related_name="+",
                to="bionty.source",
            ),
        ),
        migrations.AlterField(
            model_name="organism",
            name="synonyms",
            field=lamindb.base.fields.TextField(blank=True, default=None, null=True),
        ),
        migrations.AlterField(
            model_name="organism",
            name="uid",
            field=lamindb.base.fields.CharField(
                blank=True, default=bionty.ids.ontology, max_length=8, unique=True
            ),
        ),
        migrations.AlterField(
            model_name="organism",
            name="updated_at",
            field=lamindb.base.fields.DateTimeField(auto_now=True, db_index=True),
        ),
        migrations.AlterField(
            model_name="pathway",
            name="abbr",
            field=lamindb.base.fields.CharField(
                blank=True,
                db_index=True,
                default=None,
                max_length=32,
                null=True,
                unique=True,
            ),
        ),
        migrations.AlterField(
            model_name="pathway",
            name="created_at",
            field=lamindb.base.fields.DateTimeField(auto_now_add=True, db_index=True),
        ),
        migrations.AlterField(
            model_name="pathway",
            name="created_by",
            field=lamindb.base.fields.ForeignKey(
                blank=True,
                default=lamindb.base.users.current_user_id,
                on_delete=django.db.models.deletion.PROTECT,
                related_name="+",
                to="lamindb.user",
            ),
        ),
        migrations.AlterField(
            model_name="pathway",
            name="description",
            field=lamindb.base.fields.TextField(blank=True, default=None, null=True),
        ),
        migrations.AlterField(
            model_name="pathway",
            name="name",
            field=lamindb.base.fields.CharField(
                blank=True, db_index=True, default=None, max_length=256
            ),
        ),
        migrations.AlterField(
            model_name="pathway",
            name="ontology_id",
            field=lamindb.base.fields.CharField(
                blank=True, db_index=True, default=None, max_length=32, null=True
            ),
        ),
        migrations.AlterField(
            model_name="pathway",
            name="run",
            field=lamindb.base.fields.ForeignKey(
                blank=True,
                default=lamindb.models.current_run,
                null=True,
                on_delete=django.db.models.deletion.PROTECT,
                related_name="+",
                to="lamindb.run",
            ),
        ),
        migrations.AlterField(
            model_name="pathway",
            name="source",
            field=lamindb.base.fields.ForeignKey(
                blank=True,
                null=True,
                on_delete=django.db.models.deletion.PROTECT,
                related_name="+",
                to="bionty.source",
            ),
        ),
        migrations.AlterField(
            model_name="pathway",
            name="synonyms",
            field=lamindb.base.fields.TextField(blank=True, default=None, null=True),
        ),
        migrations.AlterField(
            model_name="pathway",
            name="uid",
            field=lamindb.base.fields.CharField(
                blank=True, default=bionty.ids.ontology, max_length=8, unique=True
            ),
        ),
        migrations.AlterField(
            model_name="pathway",
            name="updated_at",
            field=lamindb.base.fields.DateTimeField(auto_now=True, db_index=True),
        ),
        migrations.AlterField(
            model_name="phenotype",
            name="abbr",
            field=lamindb.base.fields.CharField(
                blank=True,
                db_index=True,
                default=None,
                max_length=32,
                null=True,
                unique=True,
            ),
        ),
        migrations.AlterField(
            model_name="phenotype",
            name="created_at",
            field=lamindb.base.fields.DateTimeField(auto_now_add=True, db_index=True),
        ),
        migrations.AlterField(
            model_name="phenotype",
            name="created_by",
            field=lamindb.base.fields.ForeignKey(
                blank=True,
                default=lamindb.base.users.current_user_id,
                on_delete=django.db.models.deletion.PROTECT,
                related_name="+",
                to="lamindb.user",
            ),
        ),
        migrations.AlterField(
            model_name="phenotype",
            name="description",
            field=lamindb.base.fields.TextField(blank=True, default=None, null=True),
        ),
        migrations.AlterField(
            model_name="phenotype",
            name="name",
            field=lamindb.base.fields.CharField(
                blank=True, db_index=True, default=None, max_length=256
            ),
        ),
        migrations.AlterField(
            model_name="phenotype",
            name="ontology_id",
            field=lamindb.base.fields.CharField(
                blank=True, db_index=True, default=None, max_length=32, null=True
            ),
        ),
        migrations.AlterField(
            model_name="phenotype",
            name="run",
            field=lamindb.base.fields.ForeignKey(
                blank=True,
                default=lamindb.models.current_run,
                null=True,
                on_delete=django.db.models.deletion.PROTECT,
                related_name="+",
                to="lamindb.run",
            ),
        ),
        migrations.AlterField(
            model_name="phenotype",
            name="source",
            field=lamindb.base.fields.ForeignKey(
                blank=True,
                null=True,
                on_delete=django.db.models.deletion.PROTECT,
                related_name="+",
                to="bionty.source",
            ),
        ),
        migrations.AlterField(
            model_name="phenotype",
            name="synonyms",
            field=lamindb.base.fields.TextField(blank=True, default=None, null=True),
        ),
        migrations.AlterField(
            model_name="phenotype",
            name="uid",
            field=lamindb.base.fields.CharField(
                blank=True, default=bionty.ids.ontology, max_length=8, unique=True
            ),
        ),
        migrations.AlterField(
            model_name="phenotype",
            name="updated_at",
            field=lamindb.base.fields.DateTimeField(auto_now=True, db_index=True),
        ),
        migrations.AlterField(
            model_name="protein",
            name="created_at",
            field=lamindb.base.fields.DateTimeField(auto_now_add=True, db_index=True),
        ),
        migrations.AlterField(
            model_name="protein",
            name="created_by",
            field=lamindb.base.fields.ForeignKey(
                blank=True,
                default=lamindb.base.users.current_user_id,
                on_delete=django.db.models.deletion.PROTECT,
                related_name="+",
                to="lamindb.user",
            ),
        ),
        migrations.AlterField(
            model_name="protein",
            name="description",
            field=lamindb.base.fields.TextField(blank=True, default=None, null=True),
        ),
        migrations.AlterField(
            model_name="protein",
            name="ensembl_gene_ids",
            field=lamindb.base.fields.TextField(blank=True, default=None, null=True),
        ),
        migrations.AlterField(
            model_name="protein",
            name="gene_symbol",
            field=lamindb.base.fields.CharField(
                blank=True, db_index=True, default=None, max_length=256, null=True
            ),
        ),
        migrations.AlterField(
            model_name="protein",
            name="length",
            field=lamindb.base.fields.BigIntegerField(
                blank=True, db_index=True, null=True
            ),
        ),
        migrations.AlterField(
            model_name="protein",
            name="name",
            field=lamindb.base.fields.CharField(
                blank=True, db_index=True, default=None, max_length=256, null=True
            ),
        ),
        migrations.AlterField(
            model_name="protein",
            name="organism",
            field=lamindb.base.fields.ForeignKey(
                blank=True,
                default=None,
                on_delete=django.db.models.deletion.PROTECT,
                related_name="proteins",
                to="bionty.organism",
            ),
        ),
        migrations.AlterField(
            model_name="protein",
            name="run",
            field=lamindb.base.fields.ForeignKey(
                blank=True,
                default=lamindb.models.current_run,
                null=True,
                on_delete=django.db.models.deletion.PROTECT,
                related_name="+",
                to="lamindb.run",
            ),
        ),
        migrations.AlterField(
            model_name="protein",
            name="source",
            field=lamindb.base.fields.ForeignKey(
                blank=True,
                null=True,
                on_delete=django.db.models.deletion.PROTECT,
                related_name="+",
                to="bionty.source",
            ),
        ),
        migrations.AlterField(
            model_name="protein",
            name="synonyms",
            field=lamindb.base.fields.TextField(blank=True, default=None, null=True),
        ),
        migrations.AlterField(
            model_name="protein",
            name="uid",
            field=lamindb.base.fields.CharField(
                blank=True, default=bionty.ids.protein, max_length=12, unique=True
            ),
        ),
        migrations.AlterField(
            model_name="protein",
            name="uniprotkb_id",
            field=lamindb.base.fields.CharField(
                blank=True,
                db_index=True,
                default=None,
                max_length=10,
                null=True,
                unique=True,
            ),
        ),
        migrations.AlterField(
            model_name="protein",
            name="updated_at",
            field=lamindb.base.fields.DateTimeField(auto_now=True, db_index=True),
        ),
        migrations.AlterField(
            model_name="source",
            name="created_at",
            field=lamindb.base.fields.DateTimeField(auto_now_add=True, db_index=True),
        ),
        migrations.AlterField(
            model_name="source",
            name="created_by",
            field=lamindb.base.fields.ForeignKey(
                blank=True,
                default=lamindb.base.users.current_user_id,
                on_delete=django.db.models.deletion.PROTECT,
                related_name="+",
                to="lamindb.user",
            ),
        ),
        migrations.AlterField(
            model_name="source",
            name="currently_used",
            field=lamindb.base.fields.BooleanField(
                blank=True, db_index=True, default=False
            ),
        ),
        migrations.AlterField(
            model_name="source",
            name="dataframe_artifact",
            field=lamindb.base.fields.ForeignKey(
                blank=True,
                default=None,
                null=True,
                on_delete=django.db.models.deletion.PROTECT,
                related_name="_source_dataframe_of",
                to="lamindb.artifact",
            ),
        ),
        migrations.AlterField(
            model_name="source",
            name="description",
            field=lamindb.base.fields.TextField(
                blank=True, db_index=True, default=None
            ),
        ),
        migrations.AlterField(
            model_name="source",
            name="entity",
            field=lamindb.base.fields.CharField(
                blank=True, db_index=True, default=None, max_length=256
            ),
        ),
        migrations.AlterField(
            model_name="source",
            name="in_db",
            field=lamindb.base.fields.BooleanField(
                blank=True, db_index=True, default=False
            ),
        ),
        migrations.AlterField(
            model_name="source",
            name="md5",
            field=lamindb.base.fields.TextField(blank=True, default=None, null=True),
        ),
        migrations.AlterField(
            model_name="source",
            name="name",
            field=lamindb.base.fields.CharField(
                blank=True, db_index=True, default=None, max_length=64
            ),
        ),
        migrations.AlterField(
            model_name="source",
            name="organism",
            field=lamindb.base.fields.CharField(
                blank=True, db_index=True, default=None, max_length=64
            ),
        ),
        migrations.AlterField(
            model_name="source",
            name="run",
            field=lamindb.base.fields.ForeignKey(
                blank=True,
                default=lamindb.models.current_run,
                null=True,
                on_delete=django.db.models.deletion.PROTECT,
                related_name="+",
                to="lamindb.run",
            ),
        ),
        migrations.AlterField(
            model_name="source",
            name="source_website",
            field=lamindb.base.fields.TextField(blank=True, default=None, null=True),
        ),
        migrations.AlterField(
            model_name="source",
            name="uid",
            field=lamindb.base.fields.CharField(
                blank=True, default=bionty.ids.source, max_length=4, unique=True
            ),
        ),
        migrations.AlterField(
            model_name="source",
            name="updated_at",
            field=lamindb.base.fields.DateTimeField(auto_now=True, db_index=True),
        ),
        migrations.AlterField(
            model_name="source",
            name="url",
            field=lamindb.base.fields.TextField(blank=True, default=None, null=True),
        ),
        migrations.AlterField(
            model_name="source",
            name="version",
            field=lamindb.base.fields.CharField(
                blank=True, db_index=True, default=None, max_length=64
            ),
        ),
        migrations.AlterField(
            model_name="tissue",
            name="abbr",
            field=lamindb.base.fields.CharField(
                blank=True,
                db_index=True,
                default=None,
                max_length=32,
                null=True,
                unique=True,
            ),
        ),
        migrations.AlterField(
            model_name="tissue",
            name="created_at",
            field=lamindb.base.fields.DateTimeField(auto_now_add=True, db_index=True),
        ),
        migrations.AlterField(
            model_name="tissue",
            name="created_by",
            field=lamindb.base.fields.ForeignKey(
                blank=True,
                default=lamindb.base.users.current_user_id,
                on_delete=django.db.models.deletion.PROTECT,
                related_name="+",
                to="lamindb.user",
            ),
        ),
        migrations.AlterField(
            model_name="tissue",
            name="description",
            field=lamindb.base.fields.TextField(blank=True, default=None, null=True),
        ),
        migrations.AlterField(
            model_name="tissue",
            name="name",
            field=lamindb.base.fields.CharField(
                blank=True, db_index=True, default=None, max_length=256
            ),
        ),
        migrations.AlterField(
            model_name="tissue",
            name="ontology_id",
            field=lamindb.base.fields.CharField(
                blank=True, db_index=True, default=None, max_length=32, null=True
            ),
        ),
        migrations.AlterField(
            model_name="tissue",
            name="run",
            field=lamindb.base.fields.ForeignKey(
                blank=True,
                default=lamindb.models.current_run,
                null=True,
                on_delete=django.db.models.deletion.PROTECT,
                related_name="+",
                to="lamindb.run",
            ),
        ),
        migrations.AlterField(
            model_name="tissue",
            name="source",
            field=lamindb.base.fields.ForeignKey(
                blank=True,
                null=True,
                on_delete=django.db.models.deletion.PROTECT,
                related_name="+",
                to="bionty.source",
            ),
        ),
        migrations.AlterField(
            model_name="tissue",
            name="synonyms",
            field=lamindb.base.fields.TextField(blank=True, default=None, null=True),
        ),
        migrations.AlterField(
            model_name="tissue",
            name="uid",
            field=lamindb.base.fields.CharField(
                blank=True, default=bionty.ids.ontology, max_length=8, unique=True
            ),
        ),
        migrations.AlterField(
            model_name="tissue",
            name="updated_at",
            field=lamindb.base.fields.DateTimeField(auto_now=True, db_index=True),
        ),
    ]



import bioregistry
from bionty.base.dev._handle_sources import parse_sources_yaml
from rich import print

sources = parse_sources_yaml()
latest_versions = (
    sources.groupby("source")["version"]
    .apply(
        lambda version: version.iloc[
            version.astype(str).str.replace(".", "").str.isdigit().argmax()
        ]
    )
    .reset_index()
)
latest_versions_dict = latest_versions.set_index("source")["version"].to_dict()

new_latest_versions: dict[str, str] = {}
for source, current_latest_version in latest_versions_dict.items():
    bioregistry_version = bioregistry.get_version(source)
    if bioregistry_version:
        if bioregistry_version > current_latest_version:
            new_latest_versions[source] = bioregistry_version

if len(new_latest_versions) != 0:
    for source, version in new_latest_versions.items():
        print(
            f"[bold blue]Source: [green]{source}[blue] has a more recent version:"
            f" [green]{version}"
        )
    raise AssertionError(
        f"{len(new_latest_versions.keys())} databases have more recent versions."
    )



from __future__ import annotations

from typing import Literal

import pandas as pd

from bionty.base._public_ontology import PublicOntology
from bionty.base.dev._io import s3_bionty_assets
from bionty.base.entities._shared_docstrings import _doc_params, organism_removed


@_doc_params(doc_entities=organism_removed)
class Organism(PublicOntology):
    """Organism.

    1. Organism ontology
    https://www.ensembl.org/index.html

    Args:
        {doc_entities}
    """

    def __init__(
        self,
        taxa: Literal["vertebrates", "bacteria", "fungi", "metazoa", "plants", "all"]
        | None = None,
        source: Literal["ensembl", "ncbitaxon"] | None = None,
        version: Literal[
            "2023-06-20",
            "release-57",
            "release-108",
            "release-109",
            "release-110",
            "release-111",
            "release-112",
        ]
        | None = None,
        **kwargs,
    ):
        # To support the organism kwarg being passed in getattr access in other parts of the code
        # https://github.com/laminlabs/bionty/issues/163
        if "organism" in kwargs and taxa is None:
            taxa = kwargs.pop("organism")
        super().__init__(organism=taxa, source=source, version=version, **kwargs)

    def _load_df(self) -> pd.DataFrame:
        if self.source == "ensembl":
            if not self._local_parquet_path.exists():
                # try to download from s3
                s3_bionty_assets(
                    filename=self._parquet_filename,
                    assets_base_url="s3://bionty-assets",
                    localpath=self._local_parquet_path,
                )

            # try to download from original url
            if not self._local_parquet_path.exists():
                self._url_download(self._url, self._local_ontology_path)  # type:ignore
                df = pd.read_csv(
                    self._local_ontology_path,
                    sep="\t",
                    index_col=False,  # type:ignore
                )
                df.rename(
                    columns={
                        "#name": "name",
                        "species": "scientific_name",
                        "taxonomy_id": "ontology_id",
                    },
                    inplace=True,
                )
                df["name"] = df["name"].str.lower()
                df["ontology_id"] = "NCBITaxon:" + df["ontology_id"].astype(str)
                df.to_parquet(self._local_parquet_path)
                return df
            else:
                return pd.read_parquet(self._local_parquet_path)
        else:
            return super()._load_df()

    def df(self) -> pd.DataFrame:
        """Pandas DataFrame of the ontology.

        Returns:
            A Pandas DataFrame of the ontology.

        Examples:
            >>> import bionty.base as bionty_base
            >>> bt.Organism().df()
        """
        return self._df.set_index("name")



from __future__ import annotations

from typing import TYPE_CHECKING, Literal, NamedTuple

import pandas as pd
from lamin_utils import logger

from bionty.base._public_ontology import PublicOntology
from bionty.base._settings import settings
from bionty.base.dev._io import s3_bionty_assets

from ._organism import Organism
from ._shared_docstrings import _doc_params, doc_entites

if TYPE_CHECKING:
    from collections.abc import Iterable


class MappingResult(NamedTuple):
    """Result of mapping legacy Ensembl gene IDs to current IDs.

    Attributes:
        mapped: Dictionary of successfully mapped old ensembl IDs to new ensembl IDs
        ambiguous: Dictionary of ambiguously mapped old ensembl IDs to new ensembl IDs - when
            a legacy ID maps to multiple current IDs
        unmapped: List of old ensembl IDs that couldn't be found in the current version
    """

    mapped: dict[str, str]
    ambiguous: dict[str, list[str]]
    unmapped: list[str]


@_doc_params(doc_entities=doc_entites)
class Gene(PublicOntology):
    """Gene.

    1. Ensembl
    https://www.ensembl.org/

    Args:
        {doc_entities}

    Notes:
        Biotypes: https://www.ensembl.org/info/genome/genebuild/biotypes.html
        Gene Naming: https://www.ensembl.org/info/genome/genebuild/gene_names.html
    """

    def __init__(
        self,
        organism: Literal["human", "mouse", "saccharomyces cerevisiae"] | None = None,
        source: Literal["ensembl"] | None = None,
        version: Literal["release-109", "release-110", "release-111", "release-112"]
        | None = None,
        **kwargs,
    ):
        super().__init__(
            source=source,
            version=version,
            organism=organism,
            **kwargs,
        )

    def map_legacy_ids(self, values: Iterable) -> MappingResult:
        """Convert legacy ids to current IDs.

        This only works if the legacy ensembl ID has a reference to the current ensembl ID.
        It is possible that the HGNC IDs are identical, and the genomic locations are very close,
        but a mapping may still be absent.

        Args:
            values: Legacy ensemble gene IDs of any version

        Examples:
            >>> gene = bt.base.Gene()
            >>> gene.map_legacy_genes(["ENSG00000260150", "ENSG00000260587"])
        """
        if self.source != "ensembl":
            raise NotImplementedError
        if isinstance(values, str):
            values = [values]
        ensembl = EnsemblGene(organism=self.organism, version=self.version)
        legacy_df_filename = f"df-legacy_{self.organism}__{self.source}__{self.version}__{self.__class__.__name__}.parquet"
        legacy_df_localpath = settings.dynamicdir / legacy_df_filename
        s3_bionty_assets(
            filename=legacy_df_filename,
            assets_base_url="s3://bionty-assets",
            localpath=legacy_df_localpath,
        )
        try:
            results = pd.read_parquet(legacy_df_localpath)
        except FileNotFoundError:
            raise NotImplementedError from None
        results = results[results.old_stable_id.isin(values)].copy()
        return ensembl._process_convert_result(results, values=values)


class EnsemblGene:
    def __init__(
        self,
        organism: str,
        version: str,
        taxa: Literal[
            "vertebrates", "bacteria", "fungi", "metazoa", "plants", "all"
        ] = "vertebrates",
    ) -> None:
        """Ensembl Gene mysql.

        Args:
            organism: Name of the organism
            version: Name of the ensembl DB version, e.g. "release-110"
            taxa: The taxa of the organism to fetch genes for.
        """
        self._import()
        import mysql.connector as sql
        from sqlalchemy import create_engine

        self._organism = (
            Organism(version=version, taxa=taxa).lookup().dict().get(organism)  # type:ignore
        )
        # vertebrates and plants use different ports
        if taxa == "plants":
            port = 4157
        else:
            port = 3306
        self._url = f"mysql+mysqldb://anonymous:@ensembldb.ensembl.org:{port}/{self._organism.core_db}"
        self._engine = create_engine(url=self._url)

    def _import(self):
        try:
            import mysql.connector as sql
            from sqlalchemy import create_engine
        except ModuleNotFoundError:
            raise ModuleNotFoundError(
                "To query from the Ensembl database, please run `pip install"
                " sqlalchemy mysqlclient mysql-connector-python`"
            ) from None

    def external_dbs(self):
        return pd.read_sql("SELECT * FROM external_db", con=self._engine)

    def download_df(self, external_db_names: dict | None = None) -> pd.DataFrame:
        """Fetch gene table from Ensembl mysql database.

        Args:
            external_db_names: {external database name : df column name}, see `.external_dbs()`
                Default is {"EntrezGene": "ensembl_gene_id"}.
        """
        query_core = """
        SELECT gene.stable_id, xref.display_label, gene.biotype, gene.description, external_synonym.synonym
        FROM gene
        LEFT JOIN xref ON gene.display_xref_id = xref.xref_id
        LEFT JOIN external_synonym ON gene.display_xref_id = external_synonym.xref_id
        """

        entrez_name = {"EntrezGene": "ensembl_gene_id"}
        if external_db_names is None:
            external_db_names = entrez_name
        if entrez_name not in list(external_db_names.keys()):
            external_db_names.update(entrez_name)
        external_db_names_str = ", ".join(
            [f"'{name}'" for name in external_db_names.keys()]
        )

        query_external = f"""
        SELECT gene.stable_id, object_xref.xref_id, xref.dbprimary_acc, external_db.db_name
        FROM gene
        LEFT JOIN object_xref ON gene.gene_id = object_xref.ensembl_id
        LEFT JOIN xref ON object_xref.xref_id = xref.xref_id
        LEFT JOIN external_db ON xref.external_db_id = external_db.external_db_id
        WHERE object_xref.ensembl_object_type = 'Gene' AND external_db.db_name IN ({external_db_names_str}) # noqa
        """

        # Query for the basic gene annotations:
        results_core = pd.read_sql(query_core, con=self._engine)
        logger.info("fetching records from the core DB...")

        # aggregate metadata based on ensembl stable_id
        results_core_group = results_core.groupby("stable_id").agg(
            {
                "display_label": "first",
                "biotype": "first",
                "description": "first",
                "synonym": lambda x: "|".join([i for i in set(x) if i is not None]),
            }
        )

        # Query for external ids:
        results_external = pd.read_sql(query_external, con=self._engine)
        logger.info("fetching records from the external DBs...")

        def add_external_db_column(df: pd.DataFrame, ext_db: str, df_col: str):
            # ncbi_gene_id
            ext = (
                results_external[results_external["db_name"] == ext_db]
                .drop_duplicates(["stable_id", "dbprimary_acc"])
                .drop(columns=["xref_id", "db_name"])
            )
            ext.rename(columns={"dbprimary_acc": df_col}, inplace=True)
            ext = ext.set_index("stable_id")
            dup = ext[ext.index.duplicated(keep=False)]
            if dup.shape[0] > 0:
                logger.warning(
                    f"duplicated #rows ensembl_gene_id with {df_col}: {dup.shape[0]}"
                )
            df_merge = df.merge(ext, left_index=True, right_index=True, how="outer")
            return df_merge

        df = add_external_db_column(
            df=results_core_group, ext_db="EntrezGene", df_col="ncbi_gene_id"
        )

        for ext_db, df_col in external_db_names.items():
            if ext_db == "EntrezGene":
                continue
            df = add_external_db_column(df=df, ext_db=ext_db, df_col=df_col)

        df = df.reset_index()
        df.rename(
            columns={
                "stable_id": "ensembl_gene_id",
                "display_label": "symbol",
                "synonym": "synonyms",
            },
            inplace=True,
        )
        df_res = df.loc[
            :,
            [
                "ensembl_gene_id",
                "symbol",
                "ncbi_gene_id",
                "biotype",
                "description",
                "synonyms",
            ],
        ].copy()
        for col in df.columns:
            if col not in df_res.columns:
                df_res[col] = df[col]
        df_res = df_res[~df_res["ensembl_gene_id"].isnull()]
        df_res = df_res[~df_res["ensembl_gene_id"].isna()]

        # if stable_id is not ensembl_gene_id, keep a stable_id column
        if not all(df_res["ensembl_gene_id"].str.startswith("ENS")):
            logger.warning(
                "ensembl_gene_id column not all ENS-prefixed, writing to stable_id column."
            )
            df_res.insert(0, "stable_id", df_res.pop("ensembl_gene_id"))
            df_res = df_res.sort_values("stable_id").reset_index(drop=True)
        else:
            df_res = df_res[df_res["ensembl_gene_id"].str.startswith("ENS")]
            df_res = df_res.sort_values("ensembl_gene_id").reset_index(drop=True)

        logger.success(f"downloaded Gene table containing {df_res.shape[0]} entries.")

        return df_res

    def download_legacy_ids_df(self, df: pd.DataFrame, col: str | None = None):
        col = "ensembl_gene_id" if col is None else col
        current_ids = tuple(df[col])
        results = pd.read_sql(
            "SELECT * FROM stable_id_event JOIN mapping_session USING"
            " (mapping_session_id) WHERE type = 'gene' AND new_stable_id IN"
            f" {current_ids} AND score > 0 AND old_stable_id != new_stable_id",
            con=self._engine,
        )
        return results

    def _process_convert_result(
        self,
        results: pd.DataFrame,
        values: Iterable,
    ) -> MappingResult:
        # unique mappings
        mapper = (
            results.drop_duplicates(["old_stable_id"], keep=False)
            .set_index("old_stable_id")["new_stable_id"]
            .to_dict()
        )
        # ambiguous mappings
        ambiguous = (
            results[~results["old_stable_id"].isin(mapper)][
                ["old_stable_id", "new_stable_id"]
            ]
            .groupby("old_stable_id", group_keys=False)["new_stable_id"]
            .apply(list)
            .to_dict()
        )
        # unmappables
        unmapped = set(values).difference(results["old_stable_id"])
        return MappingResult(
            mapped=mapper, ambiguous=ambiguous, unmapped=list(unmapped)
        )

    def map_legacy_ids(self, values: Iterable, df: pd.DataFrame) -> MappingResult:
        """Maps legacy gene IDs to current Ensembl gene IDs.

        Takes legacy gene IDs and maps them to current Ensembl IDs by querying the Ensembl MySQL database.
        Returns mapping results categorized as unique mappings,
        ambiguous mappings (one legacy ID maps to multiple current IDs), and unmapped IDs.

        Args:
            values: Single gene ID string or iterable of gene ID strings to map
            df: DataFrame containing current Ensembl gene IDs in 'ensembl_gene_id' column

        Example:
            >>> map_legacy_ids(['ENSG00000139618'], df)
            MappingResult(mapped={'ENSG00000139618': 'ENSG00000012048'},
                          ambiguous={},
                          unmapped=[]
                          )
        """
        if isinstance(values, str):
            legacy_genes = f"('{values}')"
            values = [values]
        else:
            legacy_genes = tuple(values)  # type:ignore
        if len(legacy_genes) == 1:
            legacy_genes = f"('{legacy_genes[0]}')"

        current_ids = tuple(df.ensembl_gene_id)
        # query the ensembl mysql db
        results = pd.read_sql(
            "SELECT * FROM stable_id_event JOIN mapping_session USING"
            " (mapping_session_id) WHERE type = 'gene' AND old_stable_id IN"
            f" {legacy_genes} AND new_stable_id IN {current_ids} AND old_stable_id !="
            " new_stable_id",
            con=self._engine,
        )
        return self._process_convert_result(results, values)



from __future__ import annotations

from typing import Literal, Optional

from bionty.base._public_ontology import PublicOntology

from ._shared_docstrings import _doc_params, doc_entites


@_doc_params(doc_entities=doc_entites)
class Tissue(PublicOntology):
    """Tissue.

    1. Uberon
    https://github.com/obophenotype/uberon

    Args:
        {doc_entities}
    """

    def __init__(
        self,
        organism: Literal["all"] | None = None,
        source: Literal["uberon"] | None = None,
        version: Literal[
            "2022-08-19",
            "2023-02-14",
            "2023-04-19",
            "2023-09-05",
            "2024-01-18",
            "2024-02-20",
            "2024-03-22",
            "2024-05-13",
            "2024-08-07",
        ]
        | None = None,
        **kwargs,
    ) -> None:
        super().__init__(
            source=source,
            version=version,
            organism=organism,
            include_id_prefixes={"uberon": ["UBERON"]},
            include_rel="part_of",
            **kwargs,
        )



# Generated by Django 5.1 on 2024-08-02 07:53

from django.db import migrations, models

import bionty.ids


class Migration(migrations.Migration):
    dependencies = [
        ("bionty", "0033_alter_artifactcellline_artifact_and_more"),
    ]

    operations = [
        migrations.AlterUniqueTogether(
            name="source", unique_together=(("entity", "name", "organism", "version"),)
        ),
    ]



# Generated by Django 5.2 on 2025-01-15 14:41

import django.db.models.functions.datetime
import lamindb.base.fields
from django.db import migrations


class Migration(migrations.Migration):
    dependencies = [
        ("bionty", "0047_lamindbv1_part5"),
    ]

    operations = [
        migrations.AlterField(
            model_name="artifactcellline",
            name="created_at",
            field=lamindb.base.fields.DateTimeField(
                blank=True,
                db_default=django.db.models.functions.datetime.Now(),
                db_index=True,
                editable=False,
            ),
        ),
        migrations.AlterField(
            model_name="artifactcellmarker",
            name="created_at",
            field=lamindb.base.fields.DateTimeField(
                blank=True,
                db_default=django.db.models.functions.datetime.Now(),
                db_index=True,
                editable=False,
            ),
        ),
        migrations.AlterField(
            model_name="artifactcelltype",
            name="created_at",
            field=lamindb.base.fields.DateTimeField(
                blank=True,
                db_default=django.db.models.functions.datetime.Now(),
                db_index=True,
                editable=False,
            ),
        ),
        migrations.AlterField(
            model_name="artifactdevelopmentalstage",
            name="created_at",
            field=lamindb.base.fields.DateTimeField(
                blank=True,
                db_default=django.db.models.functions.datetime.Now(),
                db_index=True,
                editable=False,
            ),
        ),
        migrations.AlterField(
            model_name="artifactdisease",
            name="created_at",
            field=lamindb.base.fields.DateTimeField(
                blank=True,
                db_default=django.db.models.functions.datetime.Now(),
                db_index=True,
                editable=False,
            ),
        ),
        migrations.AlterField(
            model_name="artifactethnicity",
            name="created_at",
            field=lamindb.base.fields.DateTimeField(
                blank=True,
                db_default=django.db.models.functions.datetime.Now(),
                db_index=True,
                editable=False,
            ),
        ),
        migrations.AlterField(
            model_name="artifactexperimentalfactor",
            name="created_at",
            field=lamindb.base.fields.DateTimeField(
                blank=True,
                db_default=django.db.models.functions.datetime.Now(),
                db_index=True,
                editable=False,
            ),
        ),
        migrations.AlterField(
            model_name="artifactgene",
            name="created_at",
            field=lamindb.base.fields.DateTimeField(
                blank=True,
                db_default=django.db.models.functions.datetime.Now(),
                db_index=True,
                editable=False,
            ),
        ),
        migrations.AlterField(
            model_name="artifactorganism",
            name="created_at",
            field=lamindb.base.fields.DateTimeField(
                blank=True,
                db_default=django.db.models.functions.datetime.Now(),
                db_index=True,
                editable=False,
            ),
        ),
        migrations.AlterField(
            model_name="artifactpathway",
            name="created_at",
            field=lamindb.base.fields.DateTimeField(
                blank=True,
                db_default=django.db.models.functions.datetime.Now(),
                db_index=True,
                editable=False,
            ),
        ),
        migrations.AlterField(
            model_name="artifactphenotype",
            name="created_at",
            field=lamindb.base.fields.DateTimeField(
                blank=True,
                db_default=django.db.models.functions.datetime.Now(),
                db_index=True,
                editable=False,
            ),
        ),
        migrations.AlterField(
            model_name="artifactprotein",
            name="created_at",
            field=lamindb.base.fields.DateTimeField(
                blank=True,
                db_default=django.db.models.functions.datetime.Now(),
                db_index=True,
                editable=False,
            ),
        ),
        migrations.AlterField(
            model_name="artifacttissue",
            name="created_at",
            field=lamindb.base.fields.DateTimeField(
                blank=True,
                db_default=django.db.models.functions.datetime.Now(),
                db_index=True,
                editable=False,
            ),
        ),
        migrations.AlterField(
            model_name="cellline",
            name="created_at",
            field=lamindb.base.fields.DateTimeField(
                blank=True,
                db_default=django.db.models.functions.datetime.Now(),
                db_index=True,
                editable=False,
            ),
        ),
        migrations.AlterField(
            model_name="cellline",
            name="updated_at",
            field=lamindb.base.fields.DateTimeField(
                blank=True,
                db_default=django.db.models.functions.datetime.Now(),
                db_index=True,
                editable=False,
            ),
        ),
        migrations.AlterField(
            model_name="cellmarker",
            name="created_at",
            field=lamindb.base.fields.DateTimeField(
                blank=True,
                db_default=django.db.models.functions.datetime.Now(),
                db_index=True,
                editable=False,
            ),
        ),
        migrations.AlterField(
            model_name="cellmarker",
            name="updated_at",
            field=lamindb.base.fields.DateTimeField(
                blank=True,
                db_default=django.db.models.functions.datetime.Now(),
                db_index=True,
                editable=False,
            ),
        ),
        migrations.AlterField(
            model_name="celltype",
            name="created_at",
            field=lamindb.base.fields.DateTimeField(
                blank=True,
                db_default=django.db.models.functions.datetime.Now(),
                db_index=True,
                editable=False,
            ),
        ),
        migrations.AlterField(
            model_name="celltype",
            name="updated_at",
            field=lamindb.base.fields.DateTimeField(
                blank=True,
                db_default=django.db.models.functions.datetime.Now(),
                db_index=True,
                editable=False,
            ),
        ),
        migrations.AlterField(
            model_name="developmentalstage",
            name="created_at",
            field=lamindb.base.fields.DateTimeField(
                blank=True,
                db_default=django.db.models.functions.datetime.Now(),
                db_index=True,
                editable=False,
            ),
        ),
        migrations.AlterField(
            model_name="developmentalstage",
            name="updated_at",
            field=lamindb.base.fields.DateTimeField(
                blank=True,
                db_default=django.db.models.functions.datetime.Now(),
                db_index=True,
                editable=False,
            ),
        ),
        migrations.AlterField(
            model_name="disease",
            name="created_at",
            field=lamindb.base.fields.DateTimeField(
                blank=True,
                db_default=django.db.models.functions.datetime.Now(),
                db_index=True,
                editable=False,
            ),
        ),
        migrations.AlterField(
            model_name="disease",
            name="updated_at",
            field=lamindb.base.fields.DateTimeField(
                blank=True,
                db_default=django.db.models.functions.datetime.Now(),
                db_index=True,
                editable=False,
            ),
        ),
        migrations.AlterField(
            model_name="ethnicity",
            name="created_at",
            field=lamindb.base.fields.DateTimeField(
                blank=True,
                db_default=django.db.models.functions.datetime.Now(),
                db_index=True,
                editable=False,
            ),
        ),
        migrations.AlterField(
            model_name="ethnicity",
            name="updated_at",
            field=lamindb.base.fields.DateTimeField(
                blank=True,
                db_default=django.db.models.functions.datetime.Now(),
                db_index=True,
                editable=False,
            ),
        ),
        migrations.AlterField(
            model_name="experimentalfactor",
            name="created_at",
            field=lamindb.base.fields.DateTimeField(
                blank=True,
                db_default=django.db.models.functions.datetime.Now(),
                db_index=True,
                editable=False,
            ),
        ),
        migrations.AlterField(
            model_name="experimentalfactor",
            name="updated_at",
            field=lamindb.base.fields.DateTimeField(
                blank=True,
                db_default=django.db.models.functions.datetime.Now(),
                db_index=True,
                editable=False,
            ),
        ),
        migrations.AlterField(
            model_name="gene",
            name="created_at",
            field=lamindb.base.fields.DateTimeField(
                blank=True,
                db_default=django.db.models.functions.datetime.Now(),
                db_index=True,
                editable=False,
            ),
        ),
        migrations.AlterField(
            model_name="gene",
            name="updated_at",
            field=lamindb.base.fields.DateTimeField(
                blank=True,
                db_default=django.db.models.functions.datetime.Now(),
                db_index=True,
                editable=False,
            ),
        ),
        migrations.AlterField(
            model_name="organism",
            name="created_at",
            field=lamindb.base.fields.DateTimeField(
                blank=True,
                db_default=django.db.models.functions.datetime.Now(),
                db_index=True,
                editable=False,
            ),
        ),
        migrations.AlterField(
            model_name="organism",
            name="updated_at",
            field=lamindb.base.fields.DateTimeField(
                blank=True,
                db_default=django.db.models.functions.datetime.Now(),
                db_index=True,
                editable=False,
            ),
        ),
        migrations.AlterField(
            model_name="pathway",
            name="created_at",
            field=lamindb.base.fields.DateTimeField(
                blank=True,
                db_default=django.db.models.functions.datetime.Now(),
                db_index=True,
                editable=False,
            ),
        ),
        migrations.AlterField(
            model_name="pathway",
            name="updated_at",
            field=lamindb.base.fields.DateTimeField(
                blank=True,
                db_default=django.db.models.functions.datetime.Now(),
                db_index=True,
                editable=False,
            ),
        ),
        migrations.AlterField(
            model_name="phenotype",
            name="created_at",
            field=lamindb.base.fields.DateTimeField(
                blank=True,
                db_default=django.db.models.functions.datetime.Now(),
                db_index=True,
                editable=False,
            ),
        ),
        migrations.AlterField(
            model_name="phenotype",
            name="updated_at",
            field=lamindb.base.fields.DateTimeField(
                blank=True,
                db_default=django.db.models.functions.datetime.Now(),
                db_index=True,
                editable=False,
            ),
        ),
        migrations.AlterField(
            model_name="protein",
            name="created_at",
            field=lamindb.base.fields.DateTimeField(
                blank=True,
                db_default=django.db.models.functions.datetime.Now(),
                db_index=True,
                editable=False,
            ),
        ),
        migrations.AlterField(
            model_name="protein",
            name="updated_at",
            field=lamindb.base.fields.DateTimeField(
                blank=True,
                db_default=django.db.models.functions.datetime.Now(),
                db_index=True,
                editable=False,
            ),
        ),
        migrations.AlterField(
            model_name="source",
            name="created_at",
            field=lamindb.base.fields.DateTimeField(
                blank=True,
                db_default=django.db.models.functions.datetime.Now(),
                db_index=True,
                editable=False,
            ),
        ),
        migrations.AlterField(
            model_name="source",
            name="updated_at",
            field=lamindb.base.fields.DateTimeField(
                blank=True,
                db_default=django.db.models.functions.datetime.Now(),
                db_index=True,
                editable=False,
            ),
        ),
        migrations.AlterField(
            model_name="tissue",
            name="created_at",
            field=lamindb.base.fields.DateTimeField(
                blank=True,
                db_default=django.db.models.functions.datetime.Now(),
                db_index=True,
                editable=False,
            ),
        ),
        migrations.AlterField(
            model_name="tissue",
            name="updated_at",
            field=lamindb.base.fields.DateTimeField(
                blank=True,
                db_default=django.db.models.functions.datetime.Now(),
                db_index=True,
                editable=False,
            ),
        ),
        migrations.AlterField(
            model_name="artifactcellline",
            name="created_by",
            field=lamindb.base.fields.ForeignKey(
                blank=True,
                default=lamindb.base.users.current_user_id,
                editable=False,
                on_delete=django.db.models.deletion.PROTECT,
                related_name="+",
                to="lamindb.user",
            ),
        ),
        migrations.AlterField(
            model_name="artifactcellmarker",
            name="created_by",
            field=lamindb.base.fields.ForeignKey(
                blank=True,
                default=lamindb.base.users.current_user_id,
                editable=False,
                on_delete=django.db.models.deletion.PROTECT,
                related_name="+",
                to="lamindb.user",
            ),
        ),
        migrations.AlterField(
            model_name="artifactcelltype",
            name="created_by",
            field=lamindb.base.fields.ForeignKey(
                blank=True,
                default=lamindb.base.users.current_user_id,
                editable=False,
                on_delete=django.db.models.deletion.PROTECT,
                related_name="+",
                to="lamindb.user",
            ),
        ),
        migrations.AlterField(
            model_name="artifactdevelopmentalstage",
            name="created_by",
            field=lamindb.base.fields.ForeignKey(
                blank=True,
                default=lamindb.base.users.current_user_id,
                editable=False,
                on_delete=django.db.models.deletion.PROTECT,
                related_name="+",
                to="lamindb.user",
            ),
        ),
        migrations.AlterField(
            model_name="artifactdisease",
            name="created_by",
            field=lamindb.base.fields.ForeignKey(
                blank=True,
                default=lamindb.base.users.current_user_id,
                editable=False,
                on_delete=django.db.models.deletion.PROTECT,
                related_name="+",
                to="lamindb.user",
            ),
        ),
        migrations.AlterField(
            model_name="artifactethnicity",
            name="created_by",
            field=lamindb.base.fields.ForeignKey(
                blank=True,
                default=lamindb.base.users.current_user_id,
                editable=False,
                on_delete=django.db.models.deletion.PROTECT,
                related_name="+",
                to="lamindb.user",
            ),
        ),
        migrations.AlterField(
            model_name="artifactexperimentalfactor",
            name="created_by",
            field=lamindb.base.fields.ForeignKey(
                blank=True,
                default=lamindb.base.users.current_user_id,
                editable=False,
                on_delete=django.db.models.deletion.PROTECT,
                related_name="+",
                to="lamindb.user",
            ),
        ),
        migrations.AlterField(
            model_name="artifactgene",
            name="created_by",
            field=lamindb.base.fields.ForeignKey(
                blank=True,
                default=lamindb.base.users.current_user_id,
                editable=False,
                on_delete=django.db.models.deletion.PROTECT,
                related_name="+",
                to="lamindb.user",
            ),
        ),
        migrations.AlterField(
            model_name="artifactorganism",
            name="created_by",
            field=lamindb.base.fields.ForeignKey(
                blank=True,
                default=lamindb.base.users.current_user_id,
                editable=False,
                on_delete=django.db.models.deletion.PROTECT,
                related_name="+",
                to="lamindb.user",
            ),
        ),
        migrations.AlterField(
            model_name="artifactpathway",
            name="created_by",
            field=lamindb.base.fields.ForeignKey(
                blank=True,
                default=lamindb.base.users.current_user_id,
                editable=False,
                on_delete=django.db.models.deletion.PROTECT,
                related_name="+",
                to="lamindb.user",
            ),
        ),
        migrations.AlterField(
            model_name="artifactphenotype",
            name="created_by",
            field=lamindb.base.fields.ForeignKey(
                blank=True,
                default=lamindb.base.users.current_user_id,
                editable=False,
                on_delete=django.db.models.deletion.PROTECT,
                related_name="+",
                to="lamindb.user",
            ),
        ),
        migrations.AlterField(
            model_name="artifactprotein",
            name="created_by",
            field=lamindb.base.fields.ForeignKey(
                blank=True,
                default=lamindb.base.users.current_user_id,
                editable=False,
                on_delete=django.db.models.deletion.PROTECT,
                related_name="+",
                to="lamindb.user",
            ),
        ),
        migrations.AlterField(
            model_name="artifacttissue",
            name="created_by",
            field=lamindb.base.fields.ForeignKey(
                blank=True,
                default=lamindb.base.users.current_user_id,
                editable=False,
                on_delete=django.db.models.deletion.PROTECT,
                related_name="+",
                to="lamindb.user",
            ),
        ),
        migrations.AlterField(
            model_name="cellline",
            name="created_by",
            field=lamindb.base.fields.ForeignKey(
                blank=True,
                default=lamindb.base.users.current_user_id,
                editable=False,
                on_delete=django.db.models.deletion.PROTECT,
                related_name="+",
                to="lamindb.user",
            ),
        ),
        migrations.AlterField(
            model_name="cellmarker",
            name="created_by",
            field=lamindb.base.fields.ForeignKey(
                blank=True,
                default=lamindb.base.users.current_user_id,
                editable=False,
                on_delete=django.db.models.deletion.PROTECT,
                related_name="+",
                to="lamindb.user",
            ),
        ),
        migrations.AlterField(
            model_name="celltype",
            name="created_by",
            field=lamindb.base.fields.ForeignKey(
                blank=True,
                default=lamindb.base.users.current_user_id,
                editable=False,
                on_delete=django.db.models.deletion.PROTECT,
                related_name="+",
                to="lamindb.user",
            ),
        ),
        migrations.AlterField(
            model_name="developmentalstage",
            name="created_by",
            field=lamindb.base.fields.ForeignKey(
                blank=True,
                default=lamindb.base.users.current_user_id,
                editable=False,
                on_delete=django.db.models.deletion.PROTECT,
                related_name="+",
                to="lamindb.user",
            ),
        ),
        migrations.AlterField(
            model_name="disease",
            name="created_by",
            field=lamindb.base.fields.ForeignKey(
                blank=True,
                default=lamindb.base.users.current_user_id,
                editable=False,
                on_delete=django.db.models.deletion.PROTECT,
                related_name="+",
                to="lamindb.user",
            ),
        ),
        migrations.AlterField(
            model_name="ethnicity",
            name="created_by",
            field=lamindb.base.fields.ForeignKey(
                blank=True,
                default=lamindb.base.users.current_user_id,
                editable=False,
                on_delete=django.db.models.deletion.PROTECT,
                related_name="+",
                to="lamindb.user",
            ),
        ),
        migrations.AlterField(
            model_name="experimentalfactor",
            name="created_by",
            field=lamindb.base.fields.ForeignKey(
                blank=True,
                default=lamindb.base.users.current_user_id,
                editable=False,
                on_delete=django.db.models.deletion.PROTECT,
                related_name="+",
                to="lamindb.user",
            ),
        ),
        migrations.AlterField(
            model_name="gene",
            name="created_by",
            field=lamindb.base.fields.ForeignKey(
                blank=True,
                default=lamindb.base.users.current_user_id,
                editable=False,
                on_delete=django.db.models.deletion.PROTECT,
                related_name="+",
                to="lamindb.user",
            ),
        ),
        migrations.AlterField(
            model_name="organism",
            name="created_by",
            field=lamindb.base.fields.ForeignKey(
                blank=True,
                default=lamindb.base.users.current_user_id,
                editable=False,
                on_delete=django.db.models.deletion.PROTECT,
                related_name="+",
                to="lamindb.user",
            ),
        ),
        migrations.AlterField(
            model_name="pathway",
            name="created_by",
            field=lamindb.base.fields.ForeignKey(
                blank=True,
                default=lamindb.base.users.current_user_id,
                editable=False,
                on_delete=django.db.models.deletion.PROTECT,
                related_name="+",
                to="lamindb.user",
            ),
        ),
        migrations.AlterField(
            model_name="phenotype",
            name="created_by",
            field=lamindb.base.fields.ForeignKey(
                blank=True,
                default=lamindb.base.users.current_user_id,
                editable=False,
                on_delete=django.db.models.deletion.PROTECT,
                related_name="+",
                to="lamindb.user",
            ),
        ),
        migrations.AlterField(
            model_name="protein",
            name="created_by",
            field=lamindb.base.fields.ForeignKey(
                blank=True,
                default=lamindb.base.users.current_user_id,
                editable=False,
                on_delete=django.db.models.deletion.PROTECT,
                related_name="+",
                to="lamindb.user",
            ),
        ),
        migrations.AlterField(
            model_name="source",
            name="created_by",
            field=lamindb.base.fields.ForeignKey(
                blank=True,
                default=lamindb.base.users.current_user_id,
                editable=False,
                on_delete=django.db.models.deletion.PROTECT,
                related_name="+",
                to="lamindb.user",
            ),
        ),
        migrations.AlterField(
            model_name="tissue",
            name="created_by",
            field=lamindb.base.fields.ForeignKey(
                blank=True,
                default=lamindb.base.users.current_user_id,
                editable=False,
                on_delete=django.db.models.deletion.PROTECT,
                related_name="+",
                to="lamindb.user",
            ),
        ),
    ]



from __future__ import annotations

import warnings
from typing import TYPE_CHECKING, BinaryIO

import pandas as pd
import pronto

if TYPE_CHECKING:
    from pathlib import Path


class Ontology(pronto.Ontology):
    """Interface with ontologies via pronto.

    Also see: https://pronto.readthedocs.io/en/stable/api/pronto.Ontology.html

    Args:
        handle: Path to an ontology source file.
        import_depth: The maximum depth of imports to resolve in the ontology tree.
        timeout: The timeout in seconds to use when performing network I/O.
        threads: The number of threads to use when parsing.
        url: The url of the ontology.
        prefix: Dev only -> prefix for get_term.
    """

    def __init__(
        self,
        handle: str | Path | BinaryIO | None = None,
        import_depth: int = -1,
        timeout: int = 100,
        threads: int | None = None,
        prefix: str = "",
    ) -> None:
        self._prefix = prefix
        warnings.filterwarnings("ignore", category=pronto.warnings.ProntoWarning)
        super().__init__(
            handle=handle, import_depth=import_depth, timeout=timeout, threads=threads
        )

    def get_term(self, term):
        """Search an ontology by its id."""
        try:
            return super().get_term(term)
        except KeyError:
            return super().get_term(f"{self._prefix}{term.replace(':', '_')}")

    def to_df(
        self,
        source: str | None = None,
        include_rel: str | None = None,
        include_id_prefixes: dict[str, list[str]] | None = None,
    ):
        """Convert pronto.Ontology to a DataFrame with columns id, name, parents, definition, and synonyms.

        Args:
            source: The source of the ontology terms to include. If None, all terms are included.
            include_rel: The relationship ID to include when gathering parent relationships.
                If provided, it extends the superclasses with the specified relationship.
            include_id_prefixes: A dictionary mapping sources to lists of ID prefixes.
                Only terms with IDs starting with these prefixes will be included for the specified source.

        Returns:
            A DataFrame containing the filtered ontology terms with columns:
            - ontology_id (index): The unique identifier for each term.
            - name: The name of the term.
            - definition: The definition of the term (None if not available).
            - synonyms: A pipe-separated string of exact synonyms (None if no synonyms).
            - parents: A list of parent term IDs, including superclasses and optionally the specified relationship.
        """

        def filter_include_id_prefixes(terms: pronto.ontology._OntologyTerms):
            if include_id_prefixes and source in list(include_id_prefixes.keys()):
                return list(
                    filter(
                        lambda val: any(
                            val.id.startswith(prefix)
                            for prefix in include_id_prefixes[source]  # type: ignore
                        ),
                        terms,
                    )
                )
            else:
                return terms

        if source is not None:
            prefix_list = (
                include_id_prefixes.get(source)
                if include_id_prefixes is not None
                else None
            )
        else:
            prefix_list = None

        filtered_terms = filter_include_id_prefixes(self.terms())

        df_values = []
        for term in filtered_terms:
            # skip terms without id or name
            if (not term.id) or (not term.name):
                continue

            # term definition text
            definition = None if term.definition is None else term.definition.title()

            # concatenate synonyms into a string
            synonyms = "|".join(
                [
                    synonym.description
                    for synonym in term.synonyms
                    if synonym.scope == "EXACT"
                ]
            )
            if len(synonyms) == 0:
                synonyms = None  # type:ignore

            # get 1st degree parents and additional relatonships (include_rel such as 'part_of')
            superclasses = [
                superclass.id
                for superclass in term.superclasses(
                    distance=1, with_self=False
                ).to_set()
            ]

            if include_rel is not None:
                if include_rel in [i.id for i in term.relationships]:
                    superclasses.extend(
                        [
                            superclass.id
                            for superclass in term.objects(
                                self.get_relationship(include_rel)
                            )
                        ]
                    )

            if prefix_list is not None:
                superclasses = [
                    superclass
                    for superclass in superclasses
                    if superclass.startswith(tuple(prefix_list))
                ]

            df_values.append((term.id, term.name, definition, synonyms, superclasses))

        df = pd.DataFrame(
            df_values,
            columns=["ontology_id", "name", "definition", "synonyms", "parents"],
        ).set_index("ontology_id")

        # needed to avoid erroring in .lookup()
        df["name"] = df["name"].fillna("")

        return df



from __future__ import annotations

from lamin_utils import logger

from bionty.models import Organism


class Settings:
    """Settings.

    Directly use `.settings` rather than instantiating this class yourself.
    """

    def __init__(self):
        self._organism = None

    @property
    def organism(self) -> Organism | None:
        """Default organism argument (default `None`).

        Default record to use when `organism` argument is required in `lamindb` functionality.

        Only takes effect if explicitly set!

        Examples:
            >>> bionty.settings.organism = "mouse"
            ✅ set organism: Organism(id=vado, name=mouse, taxon_id=10090, scientific_name=mus_musculus, updated_at=2023-07-21 11:37:08, source_id=CXWj, created_by_id=DzTjkKse) # noqa
        """
        return self._organism

    @organism.setter
    def organism(self, name: str | Organism):
        if isinstance(name, Organism):
            self._organism = name
        else:
            import lamindb as ln

            # do not show the validated message for organism
            verbosity = ln.settings.verbosity
            ln.settings.verbosity = 1
            organism = Organism.from_source(name=name)
            ln.settings.verbosity = verbosity
            if organism is None:
                raise ValueError(
                    f"No organism with name='{name}' is found, please create a organism record!"
                )
            if organism._state.adding:  # type:ignore
                organism.save()  # type:ignore
            logger.debug(f"set organism: {organism}")
            self._organism = organism


settings = Settings()
settings.__doc__ = """Global :class:`~bionty.core.Settings`."""



"""Basic biological entities, coupled to public ontologies [`source <https://github.com/laminlabs/bionty/blob/main/bionty/models.py>`__].

- Create records from public ontologies using `.from_source()`.
- Access public ontologies via `.public()` to search & bulk-create records.
- Use hierarchical relationships among records (`.parents`).
- Use `.synonyms` and `.abbr` to manage synonyms.
- Manage ontology versions.

Install and mount `bionty` in a new instance:

>>> pip install 'bionty'
>>> lamin init --storage <storage_name> --modules bionty

Import the package:

>>> import bionty as bt

Access public ontologies:

>>> genes = bt.Gene.public()
>>> genes.validate(["BRCA1", "TCF7"], field="symbol")

Create records from public ontologies:

>>> cell_type = bt.CellType.from_source(ontology_id="CL:0000037")
>>> cell_type.save()

View ontological hierarchy:

>>> cell_type.view_parents()

Create in-house ontologies:

>>> cell_type_new = bt.CellType(name="my new cell type")
>>> cell_type_new.save()
>>> cell_type_new.parents.add(cell_type)
>>> cell_type_new.view_parents()

Manage synonyms:

>>> cell_type_new.add_synonyms(["my cell type", "my cell"])
>>> cell_type_new.set_abbr("MCT")

Detailed guides:

- :doc:`docs:public-ontologies`
- :doc:`docs:bio-registries`

Registries:

.. autosummary::
   :toctree: .

   Organism
   Gene
   Protein
   CellMarker
   CellType
   CellLine
   Tissue
   Disease
   Pathway
   Phenotype
   ExperimentalFactor
   DevelopmentalStage
   Ethnicity

Settings:

.. autosummary::
   :toctree: .

   settings

Ontology versions:

.. autosummary::
   :toctree: .

   Source

Submodules:

.. autosummary::
   :toctree: .

   core
   base

"""

__version__ = "1.0.0"

from lamindb_setup.core._setup_bionty_sources import (
    load_bionty_sources as _load_bionty_sources,
)

from . import base, ids

base.sync_sources()

from lamindb_setup._check_setup import _check_instance_setup


def __getattr__(name):
    if name != "models":
        _check_instance_setup(from_module="bionty")
    return globals()[name]


if _check_instance_setup():
    _load_bionty_sources()

    del __getattr__  # delete so that imports work out
    from .core._settings import settings
    from .models import (
        CellLine,
        CellMarker,
        CellType,
        DevelopmentalStage,
        Disease,
        Ethnicity,
        ExperimentalFactor,
        Gene,
        Organism,
        Pathway,
        Phenotype,
        Protein,
        Source,
        Tissue,
    )

    # backward compat
    PublicSource = Source



from __future__ import annotations

from typing import Literal

from bionty.base._public_ontology import PublicOntology

from ._shared_docstrings import _doc_params, doc_entites


@_doc_params(doc_entities=doc_entites)
class CellType(PublicOntology):
    """Cell type ontologies.

    1. Cell ontology
    https://github.com/obophenotype/cell-ontology

    Args:
        {doc_entities}
    """

    def __init__(
        self,
        organism: Literal["all"] | None = None,
        source: Literal["cl"] | None = None,
        version: Literal[
            "2022-08-16",
            "2023-02-15",
            "2023-04-20",
            "2023-08-24",
            "2024-01-04",
            "2024-02-13",
            "2024-04-05",
            "2024-05-15",
            "2024-08-16",
        ]
        | None = None,
        **kwargs,
    ) -> None:
        super().__init__(
            source=source,
            version=version,
            organism=organism,
            include_id_prefixes={"cl": ["CL"]},
            **kwargs,
        )



from __future__ import annotations

from typing import Literal

from bionty.base._public_ontology import PublicOntology

from ._shared_docstrings import _doc_params, doc_entites


@_doc_params(doc_entities=doc_entites)
class Drug(PublicOntology):
    """Drug ontologies.

    1. DRON
    Edits of terms are coordinated and reviewed on:
    https://bioportal.bioontology.org/ontologies/DRON/

    2. CHEBI
    https://www.ebi.ac.uk/chebi/

    Args:
        {doc_entities}
    """

    def __init__(
        self,
        organism: Literal["all"] | None = None,
        source: Literal["dron", "chebi"] | None = None,
        version: Literal["2024-07-27", "2023-03-10", "2024-03-02", "2024-08-05"]
        | None = None,
        **kwargs,
    ) -> None:
        super().__init__(
            source=source,
            version=version,
            organism=organism,
            include_id_prefixes={"dron": ["DRON"]},
            **kwargs,
        )



import bionty.base as bt_base
import pandas as pd


def test_lamin_bfxpipeline_inspect_name():
    df = pd.DataFrame(
        index=[
            "nf-core methylseq v2.6.0",
            "Cell Ranger v8.0.0",
            "This bfx pipeline does not exist",
        ]
    )

    bfxp = bt_base.BFXPipeline(source="lamin")
    inspected_df = bfxp.inspect(df.index, field=bfxp.name, return_df=True)

    inspect = inspected_df["__validated__"].reset_index(drop=True)
    expected_series = pd.Series([True, True, False])

    assert inspect.equals(expected_series)



import bionty.base as bt_base
import pandas as pd


def test_dron_drug_inspect_name():
    df = pd.DataFrame(
        index=[
            "triflusal",
            "citrus bioflavonoids",
            "Candida albicans",
            "Hyoscyamus extract",
            "This drug does not exist",
        ]
    )

    dt = bt_base.Drug(source="dron")
    inspected_df = dt.inspect(df.index, field=dt.name, return_df=True)

    inspect = inspected_df["__validated__"].reset_index(drop=True)
    expected_series = pd.Series([True, True, True, True, False])

    assert inspect.equals(expected_series)


def test_chebi_drug_inspect_name():
    df = pd.DataFrame(
        index=[
            "navitoclax",
            "Vismione D",
            "(+)-Atherospermoline",
            "N-[(2R,3S,6R)-2-(hydroxymethyl)-6-[2-[[oxo-[4-(trifluoromethyl)anilino]methyl]amino]ethyl]-3-oxanyl]-3-pyridinecarboxamide",
            "This drug does not exist",
        ]
    )

    dt = bt_base.Drug(source="chebi")
    inspected_df = dt.inspect(df.index, field=dt.name, return_df=True)

    inspect = inspected_df["__validated__"].reset_index(drop=True)
    expected_series = pd.Series([True, True, True, True, False])

    assert inspect.equals(expected_series)


def test_chebi_chembl_id():
    dt = bt_base.Drug(source="chebi")
    assert "CHEMBL500609" in dt.df()["chembl_id"].values



"""Access to public ontologies.

`bionty.base` is the read-only interface for public ontology that underlies bionty and doesn't require a lamindb instance.

Import the package:

>>> import bionty.base as bt_base

Access public ontologies:

>>> genes = bt_base.Gene()

Get a DataFrame of all available values:

>>> genes.df()

Entities
========

Bionty base provides access to several entities, most of which are also supported by Bionty.

.. autosummary::
   :toctree: .

   Organism
   Gene
   Protein
   CellMarker
   CellType
   CellLine
   Tissue
   Disease
   Phenotype
   Pathway
   ExperimentalFactor
   DevelopmentalStage
   Drug
   Ethnicity
   BFXPipeline
   BioSample

Base class
----------

`Pronto Ontology objects <https://pronto.readthedocs.io/en/stable/api/pronto.Ontology.html>`__ can be accessed via `{entity}.to_pronto()`.

.. autosummary::
   :toctree: .

   PublicOntology
   PublicOntologyField

Ontology sources
----------------

.. autosummary::
   :toctree: .

   display_available_sources
   display_currently_used_sources
   reset_sources
   settings

Pronto Ontology
---------------

.. autosummary::
   :toctree: .

   Ontology

"""

# dynamic classes
from . import dev
from ._display_sources import display_available_sources, display_currently_used_sources
from ._ontology import Ontology

# tools
from ._public_ontology import PublicOntology, PublicOntologyField
from ._settings import settings
from ._sync_sources import sync_sources

# sources
from .dev._handle_sources import reset_sources
from .entities._bfxpipeline import BFXPipeline
from .entities._biosample import BioSample
from .entities._cellline import CellLine
from .entities._cellmarker import CellMarker
from .entities._celltype import CellType
from .entities._developmentalstage import DevelopmentalStage
from .entities._disease import Disease
from .entities._drug import Drug
from .entities._ethnicity import Ethnicity
from .entities._experimentalfactor import ExperimentalFactor
from .entities._gene import Gene
from .entities._organism import Organism
from .entities._pathway import Pathway
from .entities._phenotype import Phenotype
from .entities._protein import Protein
from .entities._tissue import Tissue
